from __future__ import annotations

from xfintech.data.common.paginate import Paginate
from xfintech.fabric.column.info import ColumnInfo
from xfintech.fabric.column.kind import ColumnKind
from xfintech.fabric.table.info import TableInfo

PROVIDER = "tushare"
SOURCE_NAME = "cashflow_vip"
URL = "https://tushare.pro/document/2?doc_id=44"
ARGS = {
    "ts_code": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "股票代码",
    },
    "start_date": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "开始日期(YYYYMMDD)",
    },
    "end_date": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "结束日期(YYYYMMDD)",
    },
    "year": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "年份(YYYY)",
    },
    "period": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "季度(1,2,3,4), 例：2025-1 表示第一季度",
    },
    "comp_type": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "公司类型(1一般工商业2银行3保险4证券)",
    },
    "report_type": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "报表类型",
    },
}

# Exported constants
NAME = "companycashflow"
KEY = "/tushare/companycashflow"
PAGINATE = Paginate(
    pagesize=1000,
    pagelimit=1000,
)
SOURCE = TableInfo(
    name=SOURCE_NAME,
    desc="A股现金流量表数据（tushare格式）",
    meta={
        "provider": PROVIDER,
        "source": SOURCE_NAME,
        "url": URL,
        "args": ARGS,
        "type": "partitioned",
        "scale": "crosssection",
    },
    columns=[
        ColumnInfo(name="ts_code", kind=ColumnKind.STRING, desc="TS股票代码"),
        ColumnInfo(name="ann_date", kind=ColumnKind.STRING, desc="公告日期"),
        ColumnInfo(name="f_ann_date", kind=ColumnKind.STRING, desc="实际公告日期"),
        ColumnInfo(name="end_date", kind=ColumnKind.STRING, desc="报告期"),
        ColumnInfo(name="comp_type", kind=ColumnKind.STRING, desc="公司类型(1一般工商业2银行3保险4证券)"),
        ColumnInfo(name="report_type", kind=ColumnKind.STRING, desc="报表类型"),
        ColumnInfo(name="end_type", kind=ColumnKind.STRING, desc="报告期类型"),
        ColumnInfo(name="update_flag", kind=ColumnKind.STRING, desc="更新标志(1最新）"),
        ColumnInfo(name="net_profit", kind=ColumnKind.FLOAT, desc="净利润"),
        ColumnInfo(name="finan_exp", kind=ColumnKind.FLOAT, desc="财务费用"),
        ColumnInfo(name="c_fr_sale_sg", kind=ColumnKind.FLOAT, desc="销售商品、提供劳务收到的现金"),
        ColumnInfo(name="recp_tax_rends", kind=ColumnKind.FLOAT, desc="收到的税费返还"),
        ColumnInfo(name="n_depos_incr_fi", kind=ColumnKind.FLOAT, desc="客户存款和同业存放款项净增加额"),
        ColumnInfo(name="n_incr_loans_cb", kind=ColumnKind.FLOAT, desc="向中央银行借款净增加额"),
        ColumnInfo(name="n_inc_borr_oth_fi", kind=ColumnKind.FLOAT, desc="向其他金融机构拆入资金净增加额"),
        ColumnInfo(name="prem_fr_orig_contr", kind=ColumnKind.FLOAT, desc="收到原保险合同保费取得的现金"),
        ColumnInfo(name="n_incr_insured_dep", kind=ColumnKind.FLOAT, desc="保户储金净增加额"),
        ColumnInfo(name="n_reinsur_prem", kind=ColumnKind.FLOAT, desc="收到再保业务现金净额"),
        ColumnInfo(name="n_incr_disp_tfa", kind=ColumnKind.FLOAT, desc="处置交易性金融资产净增加额"),
        ColumnInfo(name="ifc_cash_incr", kind=ColumnKind.FLOAT, desc="收取利息和手续费净增加额"),
        ColumnInfo(name="n_incr_disp_faas", kind=ColumnKind.FLOAT, desc="处置可供出售金融资产净增加额"),
        ColumnInfo(name="n_incr_loans_oth_bank", kind=ColumnKind.FLOAT, desc="拆入资金净增加额"),
        ColumnInfo(name="n_cap_incr_repur", kind=ColumnKind.FLOAT, desc="回购业务资金净增加额"),
        ColumnInfo(name="c_fr_oth_operate_a", kind=ColumnKind.FLOAT, desc="收到其他与经营活动有关的现金"),
        ColumnInfo(name="c_inf_fr_operate_a", kind=ColumnKind.FLOAT, desc="经营活动现金流入小计"),
        ColumnInfo(name="c_paid_goods_s", kind=ColumnKind.FLOAT, desc="购买商品、接受劳务支付的现金"),
        ColumnInfo(name="c_paid_to_for_empl", kind=ColumnKind.FLOAT, desc="支付给职工以及为职工支付的现金"),
        ColumnInfo(name="c_paid_for_taxes", kind=ColumnKind.FLOAT, desc="支付的各项税费"),
        ColumnInfo(name="n_incr_clt_loan_adv", kind=ColumnKind.FLOAT, desc="客户贷款及垫款净增加额"),
        ColumnInfo(name="n_incr_dep_cbob", kind=ColumnKind.FLOAT, desc="存放央行和同业款项净增加额"),
        ColumnInfo(name="c_pay_claims_orig_inco", kind=ColumnKind.FLOAT, desc="支付原保险合同赔付款项的现金"),
        ColumnInfo(name="pay_handling_chrg", kind=ColumnKind.FLOAT, desc="支付手续费的现金"),
        ColumnInfo(name="pay_comm_insur_plcy", kind=ColumnKind.FLOAT, desc="支付保单红利的现金"),
        ColumnInfo(name="oth_cash_pay_oper_act", kind=ColumnKind.FLOAT, desc="支付其他与经营活动有关的现金"),
        ColumnInfo(name="st_cash_out_act", kind=ColumnKind.FLOAT, desc="经营活动现金流出小计"),
        ColumnInfo(name="n_cashflow_act", kind=ColumnKind.FLOAT, desc="经营活动产生的现金流量净额"),
        ColumnInfo(name="oth_recp_ral_inv_act", kind=ColumnKind.FLOAT, desc="收到其他与投资活动有关的现金"),
        ColumnInfo(name="c_disp_withdrwl_invest", kind=ColumnKind.FLOAT, desc="收回投资收到的现金"),
        ColumnInfo(name="c_recp_return_invest", kind=ColumnKind.FLOAT, desc="取得投资收益收到的现金"),
        ColumnInfo(
            name="n_recp_disp_fiolta", kind=ColumnKind.FLOAT, desc="处置固定资产、无形资产和其他长期资产收回的现金净额"
        ),
        ColumnInfo(name="n_recp_disp_sobu", kind=ColumnKind.FLOAT, desc="处置子公司及其他营业单位收到的现金净额"),
        ColumnInfo(name="stot_inflows_inv_act", kind=ColumnKind.FLOAT, desc="投资活动现金流入小计"),
        ColumnInfo(
            name="c_pay_acq_const_fiolta", kind=ColumnKind.FLOAT, desc="购建固定资产、无形资产和其他长期资产支付的现金"
        ),
        ColumnInfo(name="c_paid_invest", kind=ColumnKind.FLOAT, desc="投资支付的现金"),
        ColumnInfo(name="n_disp_subs_oth_biz", kind=ColumnKind.FLOAT, desc="取得子公司及其他营业单位支付的现金净额"),
        ColumnInfo(name="oth_pay_ral_inv_act", kind=ColumnKind.FLOAT, desc="支付其他与投资活动有关的现金"),
        ColumnInfo(name="n_incr_pledge_loan", kind=ColumnKind.FLOAT, desc="质押贷款净增加额"),
        ColumnInfo(name="stot_out_inv_act", kind=ColumnKind.FLOAT, desc="投资活动现金流出小计"),
        ColumnInfo(name="n_cashflow_inv_act", kind=ColumnKind.FLOAT, desc="投资活动产生的现金流量净额"),
        ColumnInfo(name="c_recp_borrow", kind=ColumnKind.FLOAT, desc="取得借款收到的现金"),
        ColumnInfo(name="proc_issue_bonds", kind=ColumnKind.FLOAT, desc="发行债券收到的现金"),
        ColumnInfo(name="oth_cash_recp_ral_fnc_act", kind=ColumnKind.FLOAT, desc="收到其他与筹资活动有关的现金"),
        ColumnInfo(name="stot_cash_in_fnc_act", kind=ColumnKind.FLOAT, desc="筹资活动现金流入小计"),
        ColumnInfo(name="free_cashflow", kind=ColumnKind.FLOAT, desc="企业自由现金流量"),
        ColumnInfo(name="c_prepay_amt_borr", kind=ColumnKind.FLOAT, desc="偿还债务支付的现金"),
        ColumnInfo(name="c_pay_dist_dpcp_int_exp", kind=ColumnKind.FLOAT, desc="分配股利、利润或偿付利息支付的现金"),
        ColumnInfo(
            name="incl_dvd_profit_paid_sc_ms", kind=ColumnKind.FLOAT, desc="其中:子公司支付给少数股东的股利、利润"
        ),
        ColumnInfo(name="oth_cashpay_ral_fnc_act", kind=ColumnKind.FLOAT, desc="支付其他与筹资活动有关的现金"),
        ColumnInfo(name="stot_cashout_fnc_act", kind=ColumnKind.FLOAT, desc="筹资活动现金流出小计"),
        ColumnInfo(name="n_cash_flows_fnc_act", kind=ColumnKind.FLOAT, desc="筹资活动产生的现金流量净额"),
        ColumnInfo(name="eff_fx_flu_cash", kind=ColumnKind.FLOAT, desc="汇率变动对现金的影响"),
        ColumnInfo(name="n_incr_cash_cash_equ", kind=ColumnKind.FLOAT, desc="现金及现金等价物净增加额"),
        ColumnInfo(name="c_cash_equ_beg_period", kind=ColumnKind.FLOAT, desc="期初现金及现金等价物余额"),
        ColumnInfo(name="c_cash_equ_end_period", kind=ColumnKind.FLOAT, desc="期末现金及现金等价物余额"),
        ColumnInfo(name="c_recp_cap_contrib", kind=ColumnKind.FLOAT, desc="吸收投资收到的现金"),
        ColumnInfo(name="incl_cash_rec_saims", kind=ColumnKind.FLOAT, desc="其中:子公司吸收少数股东投资收到的现金"),
        ColumnInfo(name="uncon_invest_loss", kind=ColumnKind.FLOAT, desc="未确认投资损失"),
        ColumnInfo(name="prov_depr_assets", kind=ColumnKind.FLOAT, desc="加:资产减值准备"),
        ColumnInfo(
            name="depr_fa_coga_dpba", kind=ColumnKind.FLOAT, desc="固定资产折旧、油气资产折耗、生产性生物资产折旧"
        ),
        ColumnInfo(name="amort_intang_assets", kind=ColumnKind.FLOAT, desc="无形资产摊销"),
        ColumnInfo(name="lt_amort_deferred_exp", kind=ColumnKind.FLOAT, desc="长期待摊费用摊销"),
        ColumnInfo(name="decr_deferred_exp", kind=ColumnKind.FLOAT, desc="待摊费用减少"),
        ColumnInfo(name="incr_acc_exp", kind=ColumnKind.FLOAT, desc="预提费用增加"),
        ColumnInfo(name="loss_disp_fiolta", kind=ColumnKind.FLOAT, desc="处置固定、无形资产和其他长期资产的损失"),
        ColumnInfo(name="loss_scr_fa", kind=ColumnKind.FLOAT, desc="固定资产报废损失"),
        ColumnInfo(name="loss_fv_chg", kind=ColumnKind.FLOAT, desc="公允价值变动损失"),
        ColumnInfo(name="invest_loss", kind=ColumnKind.FLOAT, desc="投资损失"),
        ColumnInfo(name="decr_def_inc_tax_assets", kind=ColumnKind.FLOAT, desc="递延所得税资产减少"),
        ColumnInfo(name="incr_def_inc_tax_liab", kind=ColumnKind.FLOAT, desc="递延所得税负债增加"),
        ColumnInfo(name="decr_inventories", kind=ColumnKind.FLOAT, desc="存货的减少"),
        ColumnInfo(name="decr_oper_payable", kind=ColumnKind.FLOAT, desc="经营性应收项目的减少"),
        ColumnInfo(name="incr_oper_payable", kind=ColumnKind.FLOAT, desc="经营性应付项目的增加"),
        ColumnInfo(name="others", kind=ColumnKind.FLOAT, desc="其他"),
        ColumnInfo(name="im_net_cashflow_oper_act", kind=ColumnKind.FLOAT, desc="经营活动产生的现金流量净额(间接法)"),
        ColumnInfo(name="conv_debt_into_cap", kind=ColumnKind.FLOAT, desc="债务转为资本"),
        ColumnInfo(name="conv_copbonds_due_within_1y", kind=ColumnKind.FLOAT, desc="一年内到期的可转换公司债券"),
        ColumnInfo(name="fa_fnc_leases", kind=ColumnKind.FLOAT, desc="融资租入固定资产"),
        ColumnInfo(name="im_n_incr_cash_equ", kind=ColumnKind.FLOAT, desc="现金及现金等价物净增加额(间接法)"),
        ColumnInfo(name="net_dism_capital_add", kind=ColumnKind.FLOAT, desc="拆出资金净增加额"),
        ColumnInfo(name="net_cash_rece_sec", kind=ColumnKind.FLOAT, desc="代理买卖证券收到的现金净额(元)"),
        ColumnInfo(name="credit_impa_loss", kind=ColumnKind.FLOAT, desc="信用减值损失"),
        ColumnInfo(name="use_right_asset_dep", kind=ColumnKind.FLOAT, desc="使用权资产折旧"),
        ColumnInfo(name="oth_loss_asset", kind=ColumnKind.FLOAT, desc="其他资产减值损失"),
        ColumnInfo(name="end_bal_cash", kind=ColumnKind.FLOAT, desc="现金的期末余额"),
        ColumnInfo(name="beg_bal_cash", kind=ColumnKind.FLOAT, desc="减:现金的期初余额"),
        ColumnInfo(name="end_bal_cash_equ", kind=ColumnKind.FLOAT, desc="加:现金等价物的期末余额"),
        ColumnInfo(name="beg_bal_cash_equ", kind=ColumnKind.FLOAT, desc="减:现金等价物的期初余额"),
    ],
)
TARGET = TableInfo(
    name=NAME,
    desc="A股现金流量表数据（xfintech格式）",
    meta={
        "key": KEY,
        "name": NAME,
        "type": "partitioned",
        "scale": "crosssection",
    },
    columns=[
        ColumnInfo(name="code", kind=ColumnKind.STRING, desc="TS代码"),
        ColumnInfo(name="date", kind=ColumnKind.STRING, desc="报告期日期(YYYY-MM-DD)"),
        ColumnInfo(name="datecode", kind=ColumnKind.STRING, desc="报告期日期码(YYYYMMDD)"),
        ColumnInfo(name="ann_date", kind=ColumnKind.STRING, desc="公告日期"),
        ColumnInfo(name="f_ann_date", kind=ColumnKind.STRING, desc="实际公告日期"),
        ColumnInfo(name="end_date", kind=ColumnKind.STRING, desc="报告期"),
        ColumnInfo(name="comp_type", kind=ColumnKind.STRING, desc="公司类型(1一般工商业2银行3保险4证券)"),
        ColumnInfo(name="report_type", kind=ColumnKind.STRING, desc="报表类型"),
        ColumnInfo(name="end_type", kind=ColumnKind.STRING, desc="报告期类型"),
        ColumnInfo(name="update_flag", kind=ColumnKind.STRING, desc="更新标志(1最新）"),
        ColumnInfo(name="net_profit", kind=ColumnKind.FLOAT, desc="净利润"),
        ColumnInfo(name="finan_exp", kind=ColumnKind.FLOAT, desc="财务费用"),
        ColumnInfo(name="c_fr_sale_sg", kind=ColumnKind.FLOAT, desc="销售商品、提供劳务收到的现金"),
        ColumnInfo(name="recp_tax_rends", kind=ColumnKind.FLOAT, desc="收到的税费返还"),
        ColumnInfo(name="n_depos_incr_fi", kind=ColumnKind.FLOAT, desc="客户存款和同业存放款项净增加额"),
        ColumnInfo(name="n_incr_loans_cb", kind=ColumnKind.FLOAT, desc="向中央银行借款净增加额"),
        ColumnInfo(name="n_inc_borr_oth_fi", kind=ColumnKind.FLOAT, desc="向其他金融机构拆入资金净增加额"),
        ColumnInfo(name="prem_fr_orig_contr", kind=ColumnKind.FLOAT, desc="收到原保险合同保费取得的现金"),
        ColumnInfo(name="n_incr_insured_dep", kind=ColumnKind.FLOAT, desc="保户储金净增加额"),
        ColumnInfo(name="n_reinsur_prem", kind=ColumnKind.FLOAT, desc="收到再保业务现金净额"),
        ColumnInfo(name="n_incr_disp_tfa", kind=ColumnKind.FLOAT, desc="处置交易性金融资产净增加额"),
        ColumnInfo(name="ifc_cash_incr", kind=ColumnKind.FLOAT, desc="收取利息和手续费净增加额"),
        ColumnInfo(name="n_incr_disp_faas", kind=ColumnKind.FLOAT, desc="处置可供出售金融资产净增加额"),
        ColumnInfo(name="n_incr_loans_oth_bank", kind=ColumnKind.FLOAT, desc="拆入资金净增加额"),
        ColumnInfo(name="n_cap_incr_repur", kind=ColumnKind.FLOAT, desc="回购业务资金净增加额"),
        ColumnInfo(name="c_fr_oth_operate_a", kind=ColumnKind.FLOAT, desc="收到其他与经营活动有关的现金"),
        ColumnInfo(name="c_inf_fr_operate_a", kind=ColumnKind.FLOAT, desc="经营活动现金流入小计"),
        ColumnInfo(name="c_paid_goods_s", kind=ColumnKind.FLOAT, desc="购买商品、接受劳务支付的现金"),
        ColumnInfo(name="c_paid_to_for_empl", kind=ColumnKind.FLOAT, desc="支付给职工以及为职工支付的现金"),
        ColumnInfo(name="c_paid_for_taxes", kind=ColumnKind.FLOAT, desc="支付的各项税费"),
        ColumnInfo(name="n_incr_clt_loan_adv", kind=ColumnKind.FLOAT, desc="客户贷款及垫款净增加额"),
        ColumnInfo(name="n_incr_dep_cbob", kind=ColumnKind.FLOAT, desc="存放央行和同业款项净增加额"),
        ColumnInfo(name="c_pay_claims_orig_inco", kind=ColumnKind.FLOAT, desc="支付原保险合同赔付款项的现金"),
        ColumnInfo(name="pay_handling_chrg", kind=ColumnKind.FLOAT, desc="支付手续费的现金"),
        ColumnInfo(name="pay_comm_insur_plcy", kind=ColumnKind.FLOAT, desc="支付保单红利的现金"),
        ColumnInfo(name="oth_cash_pay_oper_act", kind=ColumnKind.FLOAT, desc="支付其他与经营活动有关的现金"),
        ColumnInfo(name="st_cash_out_act", kind=ColumnKind.FLOAT, desc="经营活动现金流出小计"),
        ColumnInfo(name="n_cashflow_act", kind=ColumnKind.FLOAT, desc="经营活动产生的现金流量净额"),
        ColumnInfo(name="oth_recp_ral_inv_act", kind=ColumnKind.FLOAT, desc="收到其他与投资活动有关的现金"),
        ColumnInfo(name="c_disp_withdrwl_invest", kind=ColumnKind.FLOAT, desc="收回投资收到的现金"),
        ColumnInfo(name="c_recp_return_invest", kind=ColumnKind.FLOAT, desc="取得投资收益收到的现金"),
        ColumnInfo(
            name="n_recp_disp_fiolta", kind=ColumnKind.FLOAT, desc="处置固定资产、无形资产和其他长期资产收回的现金净额"
        ),
        ColumnInfo(name="n_recp_disp_sobu", kind=ColumnKind.FLOAT, desc="处置子公司及其他营业单位收到的现金净额"),
        ColumnInfo(name="stot_inflows_inv_act", kind=ColumnKind.FLOAT, desc="投资活动现金流入小计"),
        ColumnInfo(
            name="c_pay_acq_const_fiolta", kind=ColumnKind.FLOAT, desc="购建固定资产、无形资产和其他长期资产支付的现金"
        ),
        ColumnInfo(name="c_paid_invest", kind=ColumnKind.FLOAT, desc="投资支付的现金"),
        ColumnInfo(name="n_disp_subs_oth_biz", kind=ColumnKind.FLOAT, desc="取得子公司及其他营业单位支付的现金净额"),
        ColumnInfo(name="oth_pay_ral_inv_act", kind=ColumnKind.FLOAT, desc="支付其他与投资活动有关的现金"),
        ColumnInfo(name="n_incr_pledge_loan", kind=ColumnKind.FLOAT, desc="质押贷款净增加额"),
        ColumnInfo(name="stot_out_inv_act", kind=ColumnKind.FLOAT, desc="投资活动现金流出小计"),
        ColumnInfo(name="n_cashflow_inv_act", kind=ColumnKind.FLOAT, desc="投资活动产生的现金流量净额"),
        ColumnInfo(name="c_recp_borrow", kind=ColumnKind.FLOAT, desc="取得借款收到的现金"),
        ColumnInfo(name="proc_issue_bonds", kind=ColumnKind.FLOAT, desc="发行债券收到的现金"),
        ColumnInfo(name="oth_cash_recp_ral_fnc_act", kind=ColumnKind.FLOAT, desc="收到其他与筹资活动有关的现金"),
        ColumnInfo(name="stot_cash_in_fnc_act", kind=ColumnKind.FLOAT, desc="筹资活动现金流入小计"),
        ColumnInfo(name="free_cashflow", kind=ColumnKind.FLOAT, desc="企业自由现金流量"),
        ColumnInfo(name="c_prepay_amt_borr", kind=ColumnKind.FLOAT, desc="偿还债务支付的现金"),
        ColumnInfo(name="c_pay_dist_dpcp_int_exp", kind=ColumnKind.FLOAT, desc="分配股利、利润或偿付利息支付的现金"),
        ColumnInfo(
            name="incl_dvd_profit_paid_sc_ms", kind=ColumnKind.FLOAT, desc="其中:子公司支付给少数股东的股利、利润"
        ),
        ColumnInfo(name="oth_cashpay_ral_fnc_act", kind=ColumnKind.FLOAT, desc="支付其他与筹资活动有关的现金"),
        ColumnInfo(name="stot_cashout_fnc_act", kind=ColumnKind.FLOAT, desc="筹资活动现金流出小计"),
        ColumnInfo(name="n_cash_flows_fnc_act", kind=ColumnKind.FLOAT, desc="筹资活动产生的现金流量净额"),
        ColumnInfo(name="eff_fx_flu_cash", kind=ColumnKind.FLOAT, desc="汇率变动对现金的影响"),
        ColumnInfo(name="n_incr_cash_cash_equ", kind=ColumnKind.FLOAT, desc="现金及现金等价物净增加额"),
        ColumnInfo(name="c_cash_equ_beg_period", kind=ColumnKind.FLOAT, desc="期初现金及现金等价物余额"),
        ColumnInfo(name="c_cash_equ_end_period", kind=ColumnKind.FLOAT, desc="期末现金及现金等价物余额"),
        ColumnInfo(name="c_recp_cap_contrib", kind=ColumnKind.FLOAT, desc="吸收投资收到的现金"),
        ColumnInfo(name="incl_cash_rec_saims", kind=ColumnKind.FLOAT, desc="其中:子公司吸收少数股东投资收到的现金"),
        ColumnInfo(name="uncon_invest_loss", kind=ColumnKind.FLOAT, desc="未确认投资损失"),
        ColumnInfo(name="prov_depr_assets", kind=ColumnKind.FLOAT, desc="加:资产减值准备"),
        ColumnInfo(
            name="depr_fa_coga_dpba", kind=ColumnKind.FLOAT, desc="固定资产折旧、油气资产折耗、生产性生物资产折旧"
        ),
        ColumnInfo(name="amort_intang_assets", kind=ColumnKind.FLOAT, desc="无形资产摊销"),
        ColumnInfo(name="lt_amort_deferred_exp", kind=ColumnKind.FLOAT, desc="长期待摊费用摊销"),
        ColumnInfo(name="decr_deferred_exp", kind=ColumnKind.FLOAT, desc="待摊费用减少"),
        ColumnInfo(name="incr_acc_exp", kind=ColumnKind.FLOAT, desc="预提费用增加"),
        ColumnInfo(name="loss_disp_fiolta", kind=ColumnKind.FLOAT, desc="处置固定、无形资产和其他长期资产的损失"),
        ColumnInfo(name="loss_scr_fa", kind=ColumnKind.FLOAT, desc="固定资产报废损失"),
        ColumnInfo(name="loss_fv_chg", kind=ColumnKind.FLOAT, desc="公允价值变动损失"),
        ColumnInfo(name="invest_loss", kind=ColumnKind.FLOAT, desc="投资损失"),
        ColumnInfo(name="decr_def_inc_tax_assets", kind=ColumnKind.FLOAT, desc="递延所得税资产减少"),
        ColumnInfo(name="incr_def_inc_tax_liab", kind=ColumnKind.FLOAT, desc="递延所得税负债增加"),
        ColumnInfo(name="decr_inventories", kind=ColumnKind.FLOAT, desc="存货的减少"),
        ColumnInfo(name="decr_oper_payable", kind=ColumnKind.FLOAT, desc="经营性应收项目的减少"),
        ColumnInfo(name="incr_oper_payable", kind=ColumnKind.FLOAT, desc="经营性应付项目的增加"),
        ColumnInfo(name="others", kind=ColumnKind.FLOAT, desc="其他"),
        ColumnInfo(name="im_net_cashflow_oper_act", kind=ColumnKind.FLOAT, desc="经营活动产生的现金流量净额(间接法)"),
        ColumnInfo(name="conv_debt_into_cap", kind=ColumnKind.FLOAT, desc="债务转为资本"),
        ColumnInfo(name="conv_copbonds_due_within_1y", kind=ColumnKind.FLOAT, desc="一年内到期的可转换公司债券"),
        ColumnInfo(name="fa_fnc_leases", kind=ColumnKind.FLOAT, desc="融资租入固定资产"),
        ColumnInfo(name="im_n_incr_cash_equ", kind=ColumnKind.FLOAT, desc="现金及现金等价物净增加额(间接法)"),
        ColumnInfo(name="net_dism_capital_add", kind=ColumnKind.FLOAT, desc="拆出资金净增加额"),
        ColumnInfo(name="net_cash_rece_sec", kind=ColumnKind.FLOAT, desc="代理买卖证券收到的现金净额(元)"),
        ColumnInfo(name="credit_impa_loss", kind=ColumnKind.FLOAT, desc="信用减值损失"),
        ColumnInfo(name="use_right_asset_dep", kind=ColumnKind.FLOAT, desc="使用权资产折旧"),
        ColumnInfo(name="oth_loss_asset", kind=ColumnKind.FLOAT, desc="其他资产减值损失"),
        ColumnInfo(name="end_bal_cash", kind=ColumnKind.FLOAT, desc="现金的期末余额"),
        ColumnInfo(name="beg_bal_cash", kind=ColumnKind.FLOAT, desc="减:现金的期初余额"),
        ColumnInfo(name="end_bal_cash_equ", kind=ColumnKind.FLOAT, desc="加:现金等价物的期末余额"),
        ColumnInfo(name="beg_bal_cash_equ", kind=ColumnKind.FLOAT, desc="减:现金等价物的期初余额"),
    ],
)



from unittest.mock import MagicMock, patch

import pytest

from xfintech.connect.artifact import Artifact
from xfintech.connect.common.connectref import ConnectRef

# ============================================================================
# Fixtures
# ============================================================================


@pytest.fixture
def mock_connect_ref():
    """Create a mock ConnectRef"""
    ref = MagicMock(spec=ConnectRef)
    ref.location = "s3://test-bucket/test-key.json"
    ref.describe = MagicMock(return_value={"location": ref.location})
    ref.to_dict = MagicMock(return_value={"location": ref.location})
    ref.__str__ = MagicMock(return_value=ref.location)
    return ref


@pytest.fixture
def mock_connect():
    """Create a mock Connect object"""
    connect = MagicMock()
    connect.get_object = MagicMock(return_value=b'{"key": "value"}')
    connect.put_object = MagicMock(return_value=MagicMock(location="s3://test-bucket/test-key.json"))
    return connect


@pytest.fixture
def mock_serialiser():
    """Create a mock Serialiser"""
    serialiser = MagicMock()
    serialiser.serialise = MagicMock(return_value=b'{"serialised": true}')
    return serialiser


@pytest.fixture
def mock_deserialiser():
    """Create a mock Deserialiser"""
    deserialiser = MagicMock()
    deserialiser.deserialise = MagicMock(return_value={"deserialised": True})
    return deserialiser


# ============================================================================
# Initialization Tests
# ============================================================================


def test_artifact_init_with_ref_object(mock_connect_ref):
    """Test Artifact initialization with ConnectRef object"""
    artifact = Artifact(ref=mock_connect_ref)

    assert artifact.ref == mock_connect_ref
    assert artifact.data is None
    assert artifact.meta is None


def test_artifact_init_with_ref_dict():
    """Test Artifact initialization with ref as dict"""
    ref_dict = {"location": "s3://test-bucket/test-key.json"}

    with patch("xfintech.connect.artifact.artifact.ConnectRef.from_dict") as mock_from_dict:
        mock_ref = MagicMock()
        mock_from_dict.return_value = mock_ref

        artifact = Artifact(ref=ref_dict)

        mock_from_dict.assert_called_once_with(ref_dict)
        assert artifact.ref == mock_ref


def test_artifact_init_with_data(mock_connect_ref):
    """Test Artifact initialization with data"""
    test_data = {"key": "value", "number": 123}
    artifact = Artifact(ref=mock_connect_ref, data=test_data)

    assert artifact.ref == mock_connect_ref
    assert artifact.data == test_data
    assert artifact.meta is None


def test_artifact_init_with_meta(mock_connect_ref):
    """Test Artifact initialization with metadata"""
    test_meta = {"author": "test", "version": "1.0"}
    artifact = Artifact(ref=mock_connect_ref, meta=test_meta)

    assert artifact.ref == mock_connect_ref
    assert artifact.data is None
    assert artifact.meta == test_meta
    assert artifact.meta is not test_meta  # Should be a copy


def test_artifact_init_with_all_params(mock_connect_ref):
    """Test Artifact initialization with all parameters"""
    test_data = {"key": "value"}
    test_meta = {"author": "test"}

    artifact = Artifact(ref=mock_connect_ref, data=test_data, meta=test_meta)

    assert artifact.ref == mock_connect_ref
    assert artifact.data == test_data
    assert artifact.meta == test_meta


def test_artifact_init_with_invalid_ref_type():
    """Test Artifact initialization with invalid ref type"""
    with pytest.raises(TypeError, match="Invalid type for ref"):
        Artifact(ref="invalid_string")


def test_artifact_init_with_none_ref():
    """Test Artifact initialization with None ref"""
    with pytest.raises(TypeError):
        Artifact(ref=None)


def test_artifact_meta_is_copied(mock_connect_ref):
    """Test that meta dict is copied, not referenced"""
    original_meta = {"key": "value"}
    artifact = Artifact(ref=mock_connect_ref, meta=original_meta)

    # Modify original meta
    original_meta["key"] = "changed"

    # Artifact meta should be unchanged
    assert artifact.meta["key"] == "value"


# ============================================================================
# from_dict Tests
# ============================================================================


def test_artifact_from_dict_basic():
    """Test creating Artifact from dict with minimal data"""
    ref_dict = {"location": "s3://test-bucket/test-key.json"}
    artifact_dict = {"ref": ref_dict}

    with patch("xfintech.connect.artifact.artifact.ConnectRef.from_dict") as mock_from_dict:
        mock_ref = MagicMock()
        mock_from_dict.return_value = mock_ref

        artifact = Artifact.from_dict(artifact_dict)

        assert artifact.ref == mock_ref
        assert artifact.data is None
        assert artifact.meta is None


def test_artifact_from_dict_with_data():
    """Test creating Artifact from dict with data"""
    ref_dict = {"location": "s3://test-bucket/test-key.json"}
    test_data = {"key": "value"}
    artifact_dict = {"ref": ref_dict, "data": test_data}

    with patch("xfintech.connect.artifact.artifact.ConnectRef.from_dict") as mock_from_dict:
        mock_ref = MagicMock()
        mock_from_dict.return_value = mock_ref

        artifact = Artifact.from_dict(artifact_dict)

        assert artifact.data == test_data


def test_artifact_from_dict_with_meta():
    """Test creating Artifact from dict with metadata"""
    ref_dict = {"location": "s3://test-bucket/test-key.json"}
    test_meta = {"author": "test"}
    artifact_dict = {"ref": ref_dict, "meta": test_meta}

    with patch("xfintech.connect.artifact.artifact.ConnectRef.from_dict") as mock_from_dict:
        mock_ref = MagicMock()
        mock_from_dict.return_value = mock_ref

        artifact = Artifact.from_dict(artifact_dict)

        assert artifact.meta == test_meta


def test_artifact_from_dict_missing_ref():
    """Test from_dict raises error when ref is missing"""
    artifact_dict = {"data": {"key": "value"}}

    with pytest.raises(KeyError):
        Artifact.from_dict(artifact_dict)


def test_artifact_from_dict_with_all_fields():
    """Test creating Artifact from dict with all fields"""
    ref_dict = {"location": "s3://test-bucket/test-key.json"}
    test_data = {"key": "value"}
    test_meta = {"author": "test", "version": "1.0"}
    artifact_dict = {"ref": ref_dict, "data": test_data, "meta": test_meta}

    with patch("xfintech.connect.artifact.artifact.ConnectRef.from_dict") as mock_from_dict:
        mock_ref = MagicMock()
        mock_from_dict.return_value = mock_ref

        artifact = Artifact.from_dict(artifact_dict)

        assert artifact.ref == mock_ref
        assert artifact.data == test_data
        assert artifact.meta == test_meta


# ============================================================================
# String Representation Tests
# ============================================================================


def test_artifact_str(mock_connect_ref):
    """Test __str__ method"""
    artifact = Artifact(ref=mock_connect_ref)

    result = str(artifact)

    assert result == str(mock_connect_ref)


def test_artifact_repr(mock_connect_ref):
    """Test __repr__ method"""
    artifact = Artifact(ref=mock_connect_ref)

    result = repr(artifact)

    assert "Artifact" in result
    assert str(mock_connect_ref) in result


# ============================================================================
# write Method Tests
# ============================================================================


def test_artifact_write_basic(mock_connect_ref, mock_connect):
    """Test basic write operation without serialisation"""
    test_data = {"key": "value"}
    artifact = Artifact(ref=mock_connect_ref, data=test_data)

    artifact.write(connect=mock_connect)

    mock_connect.put_object.assert_called_once_with(
        data=test_data,
        location=mock_connect_ref.location,
    )


def test_artifact_write_with_serialiser(mock_connect_ref, mock_connect, mock_serialiser):
    """Test write operation with serialiser"""
    test_data = {"key": "value"}
    artifact = Artifact(ref=mock_connect_ref, data=test_data)

    artifact.write(connect=mock_connect, serialiser=mock_serialiser, format="json")

    mock_serialiser.serialise.assert_called_once_with(
        test_data,
        format="json",
    )
    mock_connect.put_object.assert_called_once()


def test_artifact_write_no_data(mock_connect_ref, mock_connect):
    """Test write raises error when no data"""
    artifact = Artifact(ref=mock_connect_ref)

    with pytest.raises(ValueError, match="No data to write"):
        artifact.write(connect=mock_connect)


def test_artifact_write_updates_ref(mock_connect_ref, mock_connect):
    """Test write updates the ref with returned value"""
    test_data = {"key": "value"}
    artifact = Artifact(ref=mock_connect_ref, data=test_data)

    new_ref = MagicMock()
    new_ref.location = "s3://test-bucket/new-key.json"
    mock_connect.put_object.return_value = new_ref

    artifact.write(connect=mock_connect)

    assert artifact.ref == new_ref


def test_artifact_write_with_kwargs(mock_connect_ref, mock_connect):
    """Test write passes through kwargs"""
    test_data = {"key": "value"}
    artifact = Artifact(ref=mock_connect_ref, data=test_data)

    artifact.write(connect=mock_connect, custom_param="value")

    mock_connect.put_object.assert_called_once_with(
        data=test_data,
        location=mock_connect_ref.location,
        custom_param="value",
    )


def test_artifact_write_serialiser_error(mock_connect_ref, mock_connect, mock_serialiser):
    """Test write handles serialiser errors"""
    test_data = {"key": "value"}
    artifact = Artifact(ref=mock_connect_ref, data=test_data)

    mock_serialiser.serialise.side_effect = Exception("Serialisation failed")

    with pytest.raises(Exception, match="Failed to write artifact"):
        artifact.write(connect=mock_connect, serialiser=mock_serialiser, format="json")


def test_artifact_write_connect_error(mock_connect_ref, mock_connect):
    """Test write handles connection errors"""
    test_data = {"key": "value"}
    artifact = Artifact(ref=mock_connect_ref, data=test_data)

    mock_connect.put_object.side_effect = Exception("Connection failed")

    with pytest.raises(Exception, match="Failed to write artifact"):
        artifact.write(connect=mock_connect)


# ============================================================================
# read Method Tests
# ============================================================================


def test_artifact_read_basic(mock_connect_ref, mock_connect):
    """Test basic read operation without deserialisation"""
    artifact = Artifact(ref=mock_connect_ref)

    result = artifact.read(connect=mock_connect)

    mock_connect.get_object.assert_called_once_with(
        location=mock_connect_ref.location,
    )
    assert artifact.data == b'{"key": "value"}'
    assert result is artifact  # Should return self


def test_artifact_read_with_deserialiser(mock_connect_ref, mock_connect, mock_deserialiser):
    """Test read operation with deserialiser"""
    artifact = Artifact(ref=mock_connect_ref)

    artifact.read(connect=mock_connect, deserialiser=mock_deserialiser, format="json")

    mock_connect.get_object.assert_called_once()
    mock_deserialiser.deserialise.assert_called_once_with(
        b'{"key": "value"}',
        format="json",
    )
    assert artifact.data == {"deserialised": True}


def test_artifact_read_with_kwargs(mock_connect_ref, mock_connect):
    """Test read passes through kwargs"""
    artifact = Artifact(ref=mock_connect_ref)

    artifact.read(connect=mock_connect, custom_param="value")

    mock_connect.get_object.assert_called_once_with(
        location=mock_connect_ref.location,
        custom_param="value",
    )


def test_artifact_read_returns_self(mock_connect_ref, mock_connect):
    """Test read returns self for chaining"""
    artifact = Artifact(ref=mock_connect_ref)

    result = artifact.read(connect=mock_connect)

    assert result is artifact


def test_artifact_read_connect_error(mock_connect_ref, mock_connect):
    """Test read handles connection errors"""
    artifact = Artifact(ref=mock_connect_ref)

    mock_connect.get_object.side_effect = Exception("Connection failed")

    with pytest.raises(Exception, match="Failed to read artifact"):
        artifact.read(connect=mock_connect)


def test_artifact_read_deserialiser_error(mock_connect_ref, mock_connect, mock_deserialiser):
    """Test read handles deserialiser errors"""
    artifact = Artifact(ref=mock_connect_ref)

    mock_deserialiser.deserialise.side_effect = Exception("Deserialisation failed")

    with pytest.raises(Exception, match="Failed to read artifact"):
        artifact.read(connect=mock_connect, deserialiser=mock_deserialiser, format="json")


# ============================================================================
# describe Method Tests
# ============================================================================


def test_artifact_describe_basic(mock_connect_ref):
    """Test describe method with minimal data"""
    artifact = Artifact(ref=mock_connect_ref)

    result = artifact.describe()

    assert "ref" in result
    mock_connect_ref.describe.assert_called_once()


def test_artifact_describe_with_meta(mock_connect_ref):
    """Test describe includes metadata"""
    test_meta = {"author": "test", "version": "1.0"}
    artifact = Artifact(ref=mock_connect_ref, meta=test_meta)

    result = artifact.describe()

    assert "ref" in result
    assert "meta" in result
    assert result["meta"] == test_meta


def test_artifact_describe_with_data(mock_connect_ref):
    """Test describe includes data type"""
    test_data = {"key": "value"}
    artifact = Artifact(ref=mock_connect_ref, data=test_data)

    result = artifact.describe()

    assert "ref" in result
    assert "data_type" in result
    assert result["data_type"] == "dict"


def test_artifact_describe_with_all_fields(mock_connect_ref):
    """Test describe with all fields present"""
    test_data = {"key": "value"}
    test_meta = {"author": "test"}
    artifact = Artifact(ref=mock_connect_ref, data=test_data, meta=test_meta)

    result = artifact.describe()

    assert "ref" in result
    assert "meta" in result
    assert "data_type" in result


def test_artifact_describe_ref_without_method(mock_connect_ref):
    """Test describe handles ref without describe method"""
    mock_connect_ref.describe = None
    artifact = Artifact(ref=mock_connect_ref)

    result = artifact.describe()

    assert "ref" in result
    assert isinstance(result["ref"], str)


# ============================================================================
# to_dict Method Tests
# ============================================================================


def test_artifact_to_dict_basic(mock_connect_ref):
    """Test to_dict method with minimal data"""
    artifact = Artifact(ref=mock_connect_ref)

    result = artifact.to_dict()

    assert "ref" in result
    mock_connect_ref.to_dict.assert_called_once()


def test_artifact_to_dict_with_meta(mock_connect_ref):
    """Test to_dict includes metadata"""
    test_meta = {"author": "test"}
    artifact = Artifact(ref=mock_connect_ref, meta=test_meta)

    result = artifact.to_dict()

    assert "ref" in result
    assert "meta" in result
    assert result["meta"] == test_meta


def test_artifact_to_dict_with_data(mock_connect_ref):
    """Test to_dict includes data"""
    test_data = {"key": "value"}
    artifact = Artifact(ref=mock_connect_ref, data=test_data)

    result = artifact.to_dict()

    assert "ref" in result
    assert "data" in result
    assert result["data"] == test_data


def test_artifact_to_dict_round_trip(mock_connect_ref):
    """Test from_dict and to_dict round trip"""
    test_data = {"key": "value"}
    test_meta = {"author": "test"}
    artifact1 = Artifact(ref=mock_connect_ref, data=test_data, meta=test_meta)

    # Mock to_dict to return a dict
    mock_connect_ref.to_dict.return_value = {"location": "s3://test-bucket/test-key.json"}

    artifact_dict = artifact1.to_dict()

    with patch("xfintech.connect.artifact.artifact.ConnectRef.from_dict") as mock_from_dict:
        mock_from_dict.return_value = mock_connect_ref

        artifact2 = Artifact.from_dict(artifact_dict)

        assert artifact2.data == test_data
        assert artifact2.meta == test_meta


def test_artifact_to_dict_ref_without_method(mock_connect_ref):
    """Test to_dict handles ref without to_dict method"""
    mock_connect_ref.to_dict = None
    artifact = Artifact(ref=mock_connect_ref)

    result = artifact.to_dict()

    assert "ref" in result
    assert isinstance(result["ref"], str)


# ============================================================================
# Integration Tests
# ============================================================================


def test_artifact_write_read_workflow(mock_connect_ref, mock_connect, mock_serialiser, mock_deserialiser):
    """Test complete write and read workflow"""
    test_data = {"key": "value", "number": 123}

    # Write
    artifact1 = Artifact(ref=mock_connect_ref, data=test_data)
    artifact1.write(connect=mock_connect, serialiser=mock_serialiser, format="json")

    # Read
    artifact2 = Artifact(ref=mock_connect_ref)
    artifact2.read(connect=mock_connect, deserialiser=mock_deserialiser, format="json")

    assert artifact2.data == {"deserialised": True}


def test_artifact_with_metadata_workflow(mock_connect_ref):
    """Test artifact with metadata through its lifecycle"""
    test_data = {"key": "value"}
    test_meta = {"author": "test", "version": "1.0", "created": "2026-01-08"}

    artifact = Artifact(ref=mock_connect_ref, data=test_data, meta=test_meta)

    # Verify metadata is preserved
    description = artifact.describe()
    assert description["meta"] == test_meta

    # Verify metadata in dict representation
    artifact_dict = artifact.to_dict()
    assert artifact_dict["meta"] == test_meta



from __future__ import annotations

from xfintech.data.common.paginate import Paginate
from xfintech.fabric.column.info import ColumnInfo
from xfintech.fabric.column.kind import ColumnKind
from xfintech.fabric.table.info import TableInfo

PROVIDER = "baostock"
SOURCE_NAME = "query_all_stock"
URL = "http://www.baostock.com/mainContent?file=StockBasicInfoAPI.md"
ARGS = {
    "day": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "指定交易日(YYYYMMDD)，默认当前交易日",
    },
}

# Exported constants
NAME = "stock"
KEY = "/baostock/stock"
PAGINATE = Paginate(
    pagesize=10000,
    pagelimit=100,
)
SOURCE = TableInfo(
    desc="上市股票基本信息（Baostock格式）",
    meta={
        "provider": PROVIDER,
        "source": SOURCE_NAME,
        "url": URL,
        "args": ARGS,
        "type": "partitioned",
        "scale": "crosssection",
    },
    columns=[
        ColumnInfo(name="code", kind=ColumnKind.STRING, desc="TS代码"),
        ColumnInfo(name="tradeStatus", kind=ColumnKind.STRING, desc="交易状态"),
        ColumnInfo(name="code_name", kind=ColumnKind.STRING, desc="股票名称"),
    ],
)
TARGET = TableInfo(
    desc="上市公司基本信息（xfintech格式）",
    meta={
        "key": KEY,
        "name": NAME,
        "type": "partitioned",
        "scale": "crosssection",
    },
    columns=[
        ColumnInfo(name="code", kind=ColumnKind.STRING, desc="TS代码"),
        ColumnInfo(name="trade_status", kind=ColumnKind.STRING, desc="交易状态"),
        ColumnInfo(name="name", kind=ColumnKind.STRING, desc="股票名称"),
    ],
)



"""
描述:
- ConnectLike 协议的单元测试。
- 测试协议的结构、实现和运行时类型检查。
"""

from __future__ import annotations

from typing import Any

import pytest

from xfintech.connect.common.connect import ConnectLike
from xfintech.connect.common.connectref import ConnectRef

# =============================================================================
# Protocol Structure Tests
# =============================================================================


def test_connectlike_is_protocol():
    """测试 ConnectLike 是一个协议类"""
    from typing import Protocol

    assert issubclass(ConnectLike.__class__, type(Protocol))


def test_connectlike_is_runtime_checkable():
    """测试 ConnectLike 是运行时可检查的"""
    # Check for protocol marker attributes that exist in Python 3.8+
    assert hasattr(ConnectLike, "_is_protocol") or hasattr(ConnectLike, "__protocol_attrs__")


def test_connectlike_has_get_object_method():
    """测试 ConnectLike 定义了 get_object 方法"""
    assert hasattr(ConnectLike, "get_object")


def test_connectlike_has_put_object_method():
    """测试 ConnectLike 定义了 put_object 方法"""
    assert hasattr(ConnectLike, "put_object")


def test_connectlike_method_signatures():
    """测试方法签名"""
    import inspect

    # Check get_object signature
    get_sig = inspect.signature(ConnectLike.get_object)
    get_params = list(get_sig.parameters.keys())
    assert "self" in get_params
    assert "location" in get_params
    assert "kwargs" in get_params

    # Check put_object signature
    put_sig = inspect.signature(ConnectLike.put_object)
    put_params = list(put_sig.parameters.keys())
    assert "self" in put_params
    assert "data" in put_params
    assert "location" in put_params
    assert "kwargs" in put_params


# =============================================================================
# Implementation Tests - Valid Implementations
# =============================================================================


def test_valid_implementation_basic():
    """测试基本的有效实现"""

    class ValidConnect(ConnectLike):
        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b"data"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location)

    instance = ValidConnect()
    assert isinstance(instance, ConnectLike)


def test_valid_implementation_can_call_get_object():
    """测试有效实现可以调用 get_object"""

    class ValidConnect(ConnectLike):
        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b"test_data"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location)

    connect = ValidConnect()
    result = connect.get_object("test_location")
    assert result == b"test_data"


def test_valid_implementation_can_call_put_object():
    """测试有效实现可以调用 put_object"""

    class ValidConnect:
        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b"data"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location, kind="test")

    connect = ValidConnect()
    result = connect.put_object(b"test_data", "test_location")
    assert isinstance(result, ConnectRef)
    assert result.location == "test_location"
    assert result.kind == "test"


def test_valid_implementation_with_kwargs():
    """测试有效实现支持 kwargs"""

    class ValidConnect:
        def get_object(self, location: str, **kwargs: Any) -> bytes:
            encoding = kwargs.get("encoding", "utf-8")
            return f"data:{encoding}".encode()

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            kind = kwargs.get("kind", "default")
            return ConnectRef(location=location, kind=kind)

    connect = ValidConnect()

    # Test get_object with kwargs
    result = connect.get_object("location", encoding="ascii")
    assert result == b"data:ascii"

    # Test put_object with kwargs
    ref = connect.put_object(b"data", "location", kind="custom")
    assert ref.kind == "custom"


def test_valid_implementation_with_state():
    """测试包含状态的有效实现"""

    class StatefulConnect:
        def __init__(self):
            self.kind = "stateful"
            self.call_count = 0

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            self.call_count += 1
            return b"data"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            self.call_count += 1
            return ConnectRef(location=location)

    connect = StatefulConnect()
    assert isinstance(connect, ConnectLike)
    assert connect.call_count == 0

    connect.get_object("loc")
    assert connect.call_count == 1

    connect.put_object(b"data", "loc")
    assert connect.call_count == 2


def test_valid_implementation_with_extra_methods():
    """测试包含额外方法的有效实现"""

    class RichConnect:
        def __init__(self):
            self.kind = "rich"

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b"data"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location)

        def delete_object(self, location: str) -> None:
            pass

        def list_objects(self) -> list:
            return []

    connect = RichConnect()
    assert isinstance(connect, ConnectLike)


# =============================================================================
# Implementation Tests - Invalid Implementations
# =============================================================================


def test_invalid_implementation_missing_get_object():
    """测试缺少 get_object 方法的实现"""

    class InvalidConnect:
        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location)

    instance = InvalidConnect()
    assert not isinstance(instance, ConnectLike)


def test_invalid_implementation_missing_put_object():
    """测试缺少 put_object 方法的实现"""

    class InvalidConnect:
        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b"data"

    instance = InvalidConnect()
    assert not isinstance(instance, ConnectLike)


def test_invalid_implementation_missing_both():
    """测试缺少所有方法的实现"""

    class InvalidConnect:
        pass

    instance = InvalidConnect()
    assert not isinstance(instance, ConnectLike)


def test_invalid_implementation_wrong_method_names():
    """测试方法名错误的实现"""

    class InvalidConnect:
        def get(self, location: str, **kwargs: Any) -> bytes:
            return b"data"

        def put(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location)

    instance = InvalidConnect()
    assert not isinstance(instance, ConnectLike)


# =============================================================================
# Type Checking Tests
# =============================================================================


def test_isinstance_with_valid_class():
    """测试 isinstance 检查有效类"""

    class MyConnect:
        def __init__(self):
            self.kind = "test"

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b""

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location)

    assert isinstance(MyConnect(), ConnectLike)


def test_isinstance_with_invalid_class():
    """测试 isinstance 检查无效类"""

    class NotConnect:
        def __init__(self):
            self.kind = "test"

        def other_method(self):
            pass

    assert not isinstance(NotConnect(), ConnectLike)


def test_isinstance_with_builtin_types():
    """测试 isinstance 检查内置类型"""
    assert not isinstance(str(), ConnectLike)
    assert not isinstance(int(), ConnectLike)
    assert not isinstance(list(), ConnectLike)
    assert not isinstance(dict(), ConnectLike)


def test_isinstance_with_none():
    """测试 isinstance 检查 None"""
    assert not isinstance(None, ConnectLike)


# =============================================================================
# Inheritance Tests
# =============================================================================


def test_can_inherit_from_protocol():
    """测试可以继承协议"""

    class BaseConnect(ConnectLike):
        def __init__(self):
            self.kind = "test"

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b"base"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location)

    instance = BaseConnect()
    assert isinstance(instance, ConnectLike)


def test_inherited_class_can_override():
    """测试继承类可以重写方法"""

    class BaseConnect(ConnectLike):
        def __init__(self):
            self.kind = "test"

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b"base"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location, kind="base")

    class DerivedConnect(BaseConnect):
        def __init__(self):
            self.kind = "test"

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b"derived"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location, kind="derived")

    derived = DerivedConnect()
    assert isinstance(derived, ConnectLike)
    assert derived.get_object("loc") == b"derived"
    assert derived.put_object(b"data", "loc").kind == "derived"


# =============================================================================
# Real-World Usage Tests
# =============================================================================


def test_s3_like_implementation():
    """测试类似 S3 的实现"""

    class S3MockConnect:
        def __init__(self):
            self.kind = "test"
            self.storage = {}

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            if location not in self.storage:
                raise KeyError(f"Object not found: {location}")
            return self.storage[location]

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            self.storage[location] = data
            return ConnectRef(location=location, kind="s3")

    connect = S3MockConnect()
    assert isinstance(connect, ConnectLike)

    # Put and get
    ref = connect.put_object(b"test_data", "s3://bucket/key")
    assert ref.location == "s3://bucket/key"

    data = connect.get_object("s3://bucket/key")
    assert data == b"test_data"


def test_filesystem_like_implementation():
    """测试类似文件系统的实现"""

    class FileSystemConnect:
        def __init__(self):
            self.kind = "test"
            self.files = {}

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return self.files.get(location, b"")

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            self.files[location] = data
            return ConnectRef(location=location, kind="filesystem")

    connect = FileSystemConnect()
    assert isinstance(connect, ConnectLike)

    # Test operations
    ref = connect.put_object(b"file_content", "/path/to/file")
    assert ref.kind == "filesystem"

    content = connect.get_object("/path/to/file")
    assert content == b"file_content"


def test_error_handling_implementation():
    """测试包含错误处理的实现"""

    class ErrorHandlingConnect:
        def __init__(self):
            self.kind = "test"

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            if not location:
                raise ValueError("Location cannot be empty")
            return b"data"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            if not data:
                raise ValueError("Data cannot be empty")
            if not location:
                raise ValueError("Location cannot be empty")
            return ConnectRef(location=location)

    connect = ErrorHandlingConnect()
    assert isinstance(connect, ConnectLike)

    # Test error cases
    with pytest.raises(ValueError, match="Location cannot be empty"):
        connect.get_object("")

    with pytest.raises(ValueError, match="Data cannot be empty"):
        connect.put_object(b"", "location")


def test_async_style_implementation():
    """测试异步风格的实现（同步模拟）"""

    class AsyncStyleConnect:
        def __init__(self):
            self.kind = "test"
            self.pending = []

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            # Simulate async operation
            self.pending.append(("get", location))
            return b"async_data"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            # Simulate async operation
            self.pending.append(("put", location))
            return ConnectRef(location=location, kind="async")

    connect = AsyncStyleConnect()
    assert isinstance(connect, ConnectLike)

    connect.get_object("loc1")
    connect.put_object(b"data", "loc2")

    assert len(connect.pending) == 2
    assert connect.pending[0] == ("get", "loc1")
    assert connect.pending[1] == ("put", "loc2")


# =============================================================================
# Protocol Attribute Tests
# =============================================================================


def test_protocol_has_correct_attributes():
    """测试协议具有正确的属性"""
    # Check protocol defines required methods
    assert hasattr(ConnectLike, "get_object")
    assert hasattr(ConnectLike, "put_object")
    # Verify it's marked as a protocol
    assert hasattr(ConnectLike, "_is_protocol") or hasattr(ConnectLike, "__protocol_attrs__")


def test_protocol_module():
    """测试协议所属模块"""
    assert ConnectLike.__module__ == "xfintech.connect.common.connect"


def test_protocol_name():
    """测试协议名称"""
    assert ConnectLike.__name__ == "ConnectLike"


def test_protocol_has_docstring():
    """测试协议有文档字符串"""
    assert ConnectLike.__doc__ is not None
    assert len(ConnectLike.__doc__) > 0


# =============================================================================
# Edge Cases
# =============================================================================


def test_implementation_with_optional_parameters():
    """测试带可选参数的实现"""

    class OptionalConnect:
        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return b"data"

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(
                location=location,
                kind=kwargs.get("kind"),
            )

    connect = OptionalConnect()
    assert isinstance(connect, ConnectLike)

    ref = connect.put_object(b"data", "loc", kind="custom")
    assert ref.kind == "custom"


def test_implementation_with_default_values():
    """测试带默认值的实现"""

    class DefaultConnect:
        def get_object(self, location: str = "default", **kwargs: Any) -> bytes:
            return location.encode()

        def put_object(self, data: bytes = b"", location: str = "default", **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location)

    connect = DefaultConnect()
    assert isinstance(connect, ConnectLike)


def test_multiple_instances():
    """测试多个实例"""

    class SimpleConnect:
        def __init__(self, name: str):
            self.name = name

        def get_object(self, location: str, **kwargs: Any) -> bytes:
            return self.name.encode()

        def put_object(self, data: bytes, location: str, **kwargs: Any) -> ConnectRef:
            return ConnectRef(location=location, kind=self.name)

    connect1 = SimpleConnect("first")
    connect2 = SimpleConnect("second")

    assert isinstance(connect1, ConnectLike)
    assert isinstance(connect2, ConnectLike)

    assert connect1.get_object("loc") == b"first"
    assert connect2.get_object("loc") == b"second"



from __future__ import annotations

from pathlib import Path
from typing import Any, Union

from xfintech.connect.common.connect import ConnectLike
from xfintech.connect.common.connectref import ConnectRef
from xfintech.connect.common.error import ConnectFailedError, ConnectKeyError


class MacOSConnect(ConnectLike):
    """
    描述:
    - macOS 本地文件系统连接实现类。
    - 实现了 ConnectLike 协议，用于与本地文件系统进行交互。
    - 支持读取和写入本地文件。
    - 自动处理路径解析、用户目录展开和父目录创建。
    - 适用于 macOS 和其他 Unix-like 系统。

    属性:
    - kind: 连接类型标识，固定为 "macos"。

    例子:
    ```python
        from xfintech.connect.instance.macos import MacOSConnect

        connect = MacOSConnect()

        # 写入数据到本地文件
        ref = connect.put_object(b"data", "/path/to/file.txt")

        # 读取本地文件
        data = connect.get_object("/path/to/file.txt")

        # 使用 Path 对象
        from pathlib import Path
        ref = connect.put_object(b"data", Path("~/data/file.txt"))

        # 支持用户目录展开
        data = connect.get_object("~/documents/data.bin")
    ```
    """

    def __init__(self) -> None:
        self.kind: str = "macos"

    def _resolve_path(
        self,
        location: Union[str, Path],
    ) -> Path:
        if location is None:
            msg = "location cannot be None"
            raise ConnectKeyError(msg)
        if isinstance(location, Path):
            location = location
        else:
            location = Path(location)
        location = location.expanduser().resolve(strict=False)
        location.parent.mkdir(
            parents=True,
            exist_ok=True,
        )
        return location

    def __str__(self) -> str:
        return f"{self.__class__.__name__}"

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}()"

    def put_bytes(
        self,
        data: bytes,
        location: Union[str, Path],
        **kwargs: Any,
    ) -> None:
        path = self._resolve_path(location)
        try:
            with path.open("wb") as f:
                f.write(data)
        except Exception as e:
            raise ConnectFailedError(str(e)) from e

    def get_bytes(
        self,
        location: Union[str, Path],
        **kwargs: Any,
    ) -> bytes:
        path = self._resolve_path(location)
        try:
            with path.open("rb") as f:
                data = f.read()
            return data
        except Exception as e:
            raise ConnectFailedError(str(e)) from e

    def get_object(
        self,
        location: str,
        **kwargs: Any,
    ) -> bytes:
        return self.get_bytes(
            location,
            **kwargs,
        )

    def put_object(
        self,
        data: bytes,
        location: str,
        **kwargs: Any,
    ) -> ConnectRef:
        self.put_bytes(
            data,
            location,
            **kwargs,
        )
        return ConnectRef(
            location=str(location),
            kind=self.kind,
        )



from __future__ import annotations

from typing import Any, Dict, Protocol, runtime_checkable


@runtime_checkable
class JobLike(Protocol):
    """
    描述:
    - 作业协议接口，定义作业类必须实现的核心方法。
    - 使用 Protocol 实现结构化类型检查（structural typing）。
    - 任何实现这些方法的类都会被视为符合 JobLike 协议。

    方法:
    - run(): 执行作业并返回结果。
    - _run(): 内部执行逻辑，由子类实现。
    - describe(): 返回作业描述信息字典。
    - to_dict(): 返回作业完整信息字典。

    例子:
    ```python
        from xfintech.data.job.joblike import JobLike

        # 任何实现这些方法的类都符合 JobLike 协议
        class MyJob:
            def run(self) -> Any:
                return self._run()

            def _run(self) -> Any:
                return {"result": "success"}

            def describe(self) -> Dict[str, Any]:
                return {"name": "MyJob"}

            def to_dict(self) -> Dict[str, Any]:
                return {"name": "MyJob", "status": "active"}

        # 类型检查会通过
        job: JobLike = MyJob()
        result = job.run()
    ```
    """

    def run(self) -> Any: ...
    def _run(self) -> Any: ...
    def describe(self) -> Dict[str, Any]: ...
    def to_dict(self) -> Dict[str, Any]: ...



"""
Test suite for RelayClient class
Tests cover initialization, URL/secret/timeout resolution, JSON serialization
"""

import json
from typing import Any

import pytest

from xfintech.data.relay.client import RelayClient

# ============================================================================
# Helper Classes for Testing
# ============================================================================


class ConcreteRelayClient(RelayClient):
    """Concrete implementation for testing"""

    def call(self) -> Any:
        return {"status": "success"}


class CustomRelayClient(RelayClient):
    """Custom client with overridden call"""

    def __init__(self, url: str, secret: str, timeout: int = None, **kwargs):
        super().__init__(url, secret, timeout, **kwargs)
        self.custom_attr = kwargs.get("custom_attr", "default")

    def call(self) -> dict:
        return {"url": self.url, "timeout": self.timeout, "custom": self.custom_attr}


# ============================================================================
# Initialization Tests
# ============================================================================


def test_relayclient_init_basic():
    """Test RelayClient basic initialization"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="test-secret")

    assert client.url == "https://relay.example.com"
    assert client.secret == "test-secret"
    assert client.timeout == RelayClient.DEFAULT_TIMEOUT


def test_relayclient_init_with_timeout():
    """Test RelayClient initialization with custom timeout"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="test-secret", timeout=120)

    assert client.timeout == 120


def test_relayclient_init_with_kwargs():
    """Test RelayClient initialization accepts kwargs"""
    client = CustomRelayClient(url="https://relay.example.com", secret="test-secret", custom_attr="custom_value")

    assert client.custom_attr == "custom_value"


def test_relayclient_init_strips_trailing_slash():
    """Test URL trailing slash is removed"""
    client = ConcreteRelayClient(url="https://relay.example.com/", secret="test-secret")

    assert client.url == "https://relay.example.com"


def test_relayclient_init_multiple_trailing_slashes():
    """Test multiple trailing slashes are removed"""
    client = ConcreteRelayClient(url="https://relay.example.com///", secret="test-secret")

    assert client.url == "https://relay.example.com"


def test_relayclient_init_strips_secret_whitespace():
    """Test secret whitespace is stripped"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="  test-secret  ")

    assert client.secret == "test-secret"


def test_relayclient_default_timeout_constant():
    """Test DEFAULT_TIMEOUT constant value"""
    assert RelayClient.DEFAULT_TIMEOUT == 180


# ============================================================================
# URL Resolution Tests
# ============================================================================


def test_resolve_url_valid():
    """Test _resolve_url with valid URL"""
    client = ConcreteRelayClient(url="https://api.example.com", secret="secret")

    assert client.url == "https://api.example.com"


def test_resolve_url_empty_string_raises_error():
    """Test _resolve_url raises error for empty string"""
    with pytest.raises(ValueError, match="Relay URL must be provided"):
        ConcreteRelayClient(url="", secret="secret")


def test_resolve_url_with_path():
    """Test _resolve_url preserves path"""
    client = ConcreteRelayClient(url="https://relay.example.com/api/v1", secret="secret")

    assert client.url == "https://relay.example.com/api/v1"


def test_resolve_url_with_path_and_trailing_slash():
    """Test _resolve_url preserves path but removes trailing slash"""
    client = ConcreteRelayClient(url="https://relay.example.com/api/v1/", secret="secret")

    assert client.url == "https://relay.example.com/api/v1"


def test_resolve_url_localhost():
    """Test _resolve_url with localhost"""
    client = ConcreteRelayClient(url="http://localhost:8080", secret="secret")

    assert client.url == "http://localhost:8080"


# ============================================================================
# Secret Resolution Tests
# ============================================================================


def test_resolve_secret_valid():
    """Test _resolve_secret with valid secret"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="my-secret-key")

    assert client.secret == "my-secret-key"


def test_resolve_secret_empty_string_raises_error():
    """Test _resolve_secret raises error for empty string"""
    with pytest.raises(ValueError, match="Relay secret must be provided"):
        ConcreteRelayClient(url="https://relay.example.com", secret="")


def test_resolve_secret_whitespace_only_raises_error():
    """Test _resolve_secret raises error for whitespace-only string"""
    with pytest.raises(ValueError, match="Relay secret must be provided"):
        ConcreteRelayClient(url="https://relay.example.com", secret="   ")


def test_resolve_secret_with_leading_whitespace():
    """Test _resolve_secret strips leading whitespace"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="   my-secret")

    assert client.secret == "my-secret"


def test_resolve_secret_with_trailing_whitespace():
    """Test _resolve_secret strips trailing whitespace"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="my-secret   ")

    assert client.secret == "my-secret"


def test_resolve_secret_with_internal_whitespace():
    """Test _resolve_secret preserves internal whitespace"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="my secret key")

    assert client.secret == "my secret key"


def test_resolve_secret_special_characters():
    """Test _resolve_secret accepts special characters"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret!@#$%^&*()_+-=[]{}|")

    assert client.secret == "secret!@#$%^&*()_+-=[]{}|"


# ============================================================================
# Timeout Resolution Tests
# ============================================================================


def test_resolve_timeout_none_uses_default():
    """Test _resolve_timeout uses default when None"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret", timeout=None)

    assert client.timeout == RelayClient.DEFAULT_TIMEOUT


def test_resolve_timeout_valid_integer():
    """Test _resolve_timeout with valid integer"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret", timeout=60)

    assert client.timeout == 60


def test_resolve_timeout_zero_uses_default():
    """Test _resolve_timeout uses default for zero"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret", timeout=0)

    assert client.timeout == RelayClient.DEFAULT_TIMEOUT


def test_resolve_timeout_negative_uses_default():
    """Test _resolve_timeout uses default for negative"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret", timeout=-10)

    assert client.timeout == RelayClient.DEFAULT_TIMEOUT


def test_resolve_timeout_string_raises_error():
    """Test _resolve_timeout raises error for string"""
    with pytest.raises(ValueError, match="Timeout must be an integer"):
        ConcreteRelayClient(url="https://relay.example.com", secret="secret", timeout="60")


def test_resolve_timeout_float_raises_error():
    """Test _resolve_timeout raises error for float"""
    with pytest.raises(ValueError, match="Timeout must be an integer"):
        ConcreteRelayClient(url="https://relay.example.com", secret="secret", timeout=60.5)


def test_resolve_timeout_large_value():
    """Test _resolve_timeout accepts large values"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret", timeout=3600)

    assert client.timeout == 3600


# ============================================================================
# canonical_json Method Tests
# ============================================================================


def test_canonical_json_simple_dict():
    """Test canonical_json with simple dictionary"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = {"name": "test", "value": 42}
    result = client.canonical_json(data)

    assert isinstance(result, bytes)
    assert result == b'{"name":"test","value":42}'


def test_canonical_json_sorted_keys():
    """Test canonical_json sorts keys"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = {"z": 1, "a": 2, "m": 3}
    result = client.canonical_json(data)

    assert result == b'{"a":2,"m":3,"z":1}'


def test_canonical_json_no_spaces():
    """Test canonical_json has no spaces"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = {"key1": "value1", "key2": "value2"}
    result = client.canonical_json(data)

    assert b" " not in result
    assert result == b'{"key1":"value1","key2":"value2"}'


def test_canonical_json_nested_dict():
    """Test canonical_json with nested dictionary"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = {"outer": {"inner": "value"}}
    result = client.canonical_json(data)

    assert result == b'{"outer":{"inner":"value"}}'


def test_canonical_json_list():
    """Test canonical_json with list"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = [1, 2, 3]
    result = client.canonical_json(data)

    assert result == b"[1,2,3]"


def test_canonical_json_mixed_types():
    """Test canonical_json with mixed types"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = {"string": "text", "number": 123, "float": 45.67, "bool": True, "null": None, "list": [1, 2, 3]}
    result = client.canonical_json(data)

    assert isinstance(result, bytes)
    # Verify it's valid JSON
    parsed = json.loads(result)
    assert parsed["string"] == "text"
    assert parsed["number"] == 123
    assert parsed["bool"] is True
    assert parsed["null"] is None


def test_canonical_json_unicode():
    """Test canonical_json handles unicode"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = {"name": "测试", "symbol": "€"}
    result = client.canonical_json(data)

    assert isinstance(result, bytes)
    # Should preserve unicode, not escape it
    assert "测试".encode("utf-8") in result
    assert "€".encode("utf-8") in result


def test_canonical_json_empty_dict():
    """Test canonical_json with empty dictionary"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = {}
    result = client.canonical_json(data)

    assert result == b"{}"


def test_canonical_json_empty_list():
    """Test canonical_json with empty list"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = []
    result = client.canonical_json(data)

    assert result == b"[]"


def test_canonical_json_returns_bytes():
    """Test canonical_json always returns bytes"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    result = client.canonical_json({"test": "data"})

    assert type(result) is bytes


# ============================================================================
# call Method Tests
# ============================================================================


def test_call_not_implemented():
    """Test call raises NotImplementedError in base class"""
    client = RelayClient(url="https://relay.example.com", secret="secret")

    with pytest.raises(NotImplementedError):
        client.call()


def test_call_implemented_in_subclass():
    """Test call can be implemented in subclass"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    result = client.call()

    assert result == {"status": "success"}


def test_call_custom_implementation():
    """Test call with custom implementation"""
    client = CustomRelayClient(url="https://relay.example.com", secret="secret", timeout=100, custom_attr="test")

    result = client.call()

    assert result["url"] == "https://relay.example.com"
    assert result["timeout"] == 100
    assert result["custom"] == "test"


# ============================================================================
# Integration Tests
# ============================================================================


def test_relayclient_full_initialization():
    """Test complete RelayClient initialization workflow"""
    client = ConcreteRelayClient(url="https://relay.example.com/api/v1/", secret="  my-secret-key  ", timeout=300)

    assert client.url == "https://relay.example.com/api/v1"
    assert client.secret == "my-secret-key"
    assert client.timeout == 300


def test_relayclient_json_serialization_workflow():
    """Test JSON serialization workflow"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = {"endpoint": client.url, "timeout": client.timeout, "data": {"key": "value"}}

    json_bytes = client.canonical_json(data)

    # Verify it can be deserialized
    parsed = json.loads(json_bytes)
    assert parsed["endpoint"] == client.url
    assert parsed["timeout"] == 180


def test_relayclient_multiple_instances_independent():
    """Test multiple RelayClient instances are independent"""
    client1 = ConcreteRelayClient(url="https://relay1.example.com", secret="secret1", timeout=60)

    client2 = ConcreteRelayClient(url="https://relay2.example.com", secret="secret2", timeout=120)

    assert client1.url != client2.url
    assert client1.secret != client2.secret
    assert client1.timeout != client2.timeout


def test_relayclient_protocol_compliance():
    """Test RelayClient implements RelayClientLike protocol"""
    from xfintech.data.relay.clientlike import RelayClientLike

    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    # Should have all required attributes
    assert hasattr(client, "url")
    assert hasattr(client, "secret")
    assert hasattr(client, "timeout")
    assert hasattr(client, "call")
    assert callable(client.call)

    # Should be recognized as RelayClientLike
    assert isinstance(client, RelayClientLike)


# ============================================================================
# Edge Case Tests
# ============================================================================


def test_relayclient_url_with_query_params():
    """Test URL with query parameters"""
    client = ConcreteRelayClient(url="https://relay.example.com/api?key=value", secret="secret")

    assert client.url == "https://relay.example.com/api?key=value"


def test_relayclient_url_with_fragment():
    """Test URL with fragment"""
    client = ConcreteRelayClient(url="https://relay.example.com/api#section", secret="secret")

    assert client.url == "https://relay.example.com/api#section"


def test_relayclient_very_long_secret():
    """Test with very long secret"""
    long_secret = "x" * 1000
    client = ConcreteRelayClient(url="https://relay.example.com", secret=long_secret)

    assert len(client.secret) == 1000


def test_canonical_json_large_data():
    """Test canonical_json with large data structure"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    large_data = {f"key_{i}": f"value_{i}" for i in range(100)}
    result = client.canonical_json(large_data)

    assert isinstance(result, bytes)
    parsed = json.loads(result)
    assert len(parsed) == 100


def test_canonical_json_deeply_nested():
    """Test canonical_json with deeply nested structure"""
    client = ConcreteRelayClient(url="https://relay.example.com", secret="secret")

    data = {"level1": {"level2": {"level3": {"level4": "deep"}}}}
    result = client.canonical_json(data)

    parsed = json.loads(result)
    assert parsed["level1"]["level2"]["level3"]["level4"] == "deep"



from unittest.mock import MagicMock, patch

import pandas as pd
import pytest

from xfintech.data.common.cache import Cache
from xfintech.data.common.coolant import Coolant
from xfintech.data.common.paginate import Paginate
from xfintech.data.common.params import Params
from xfintech.data.common.retry import Retry
from xfintech.data.source.tushare.session.session import Session
from xfintech.data.source.tushare.stock.stockipo.constant import (
    KEY,
    NAME,
    PAGINATE,
    SOURCE,
    TARGET,
)
from xfintech.data.source.tushare.stock.stockipo.stockipo import StockIpo

# ============================================================================
# Fixtures
# ============================================================================


@pytest.fixture
def mock_session():
    """Create a mock Tushare session"""
    session = MagicMock(spec=Session)
    session._credential = "test_token"
    session.id = "test1234"
    session.mode = "direct"
    session.relay_url = None
    session.relay_secret = None
    session.connected = True

    # Mock the connection object
    mock_connection = MagicMock()
    mock_connection.new_share = MagicMock()
    session.connection = mock_connection

    return session


@pytest.fixture
def sample_source_data():
    """Create sample source data in Tushare format"""
    return pd.DataFrame(
        {
            "ts_code": ["688001.SH", "688002.SH", "688003.SH"],
            "sub_code": ["787001", "787002", "787003"],
            "name": ["华兴源创", "睿创微纳", "天准科技"],
            "ipo_date": ["20190627", "20190628", "20190710"],
            "issue_date": ["20190710", "20190712", "20190722"],
            "amount": [40100.0, 60000.0, 45360.0],
            "market_amount": [10025.0, 15000.0, 11340.0],
            "price": [24.26, 20.00, 25.50],
            "pe": [41.08, 79.09, 58.62],
            "limit_amount": [10.025, 15.0, 11.34],
            "funds": [9.73, 12.0, 11.57],
            "ballot": [0.0424, 0.0632, 0.0587],
        }
    )


@pytest.fixture
def expected_transformed_data():
    """Create expected transformed data"""
    return pd.DataFrame(
        {
            "code": ["688001.SH", "688002.SH", "688003.SH"],
            "sub_code": ["787001", "787002", "787003"],
            "name": ["华兴源创", "睿创微纳", "天准科技"],
            "ipo_date": ["2019-06-27", "2019-06-28", "2019-07-10"],
            "ipo_datecode": ["20190627", "20190628", "20190710"],
            "issue_date": ["2019-07-10", "2019-07-12", "2019-07-22"],
            "issue_datecode": ["20190710", "20190712", "20190722"],
            "amount": [40100.0, 60000.0, 45360.0],
            "market_amount": [10025.0, 15000.0, 11340.0],
            "price": [24.26, 20.00, 25.50],
            "pe": [41.08, 79.09, 58.62],
            "limit_amount": [10.025, 15.0, 11.34],
            "funds": [9.73, 12.0, 11.57],
            "ballot": [0.0424, 0.0632, 0.0587],
        }
    )


# ============================================================================
# Initialization Tests
# ============================================================================


def test_ipo_init_basic(mock_session):
    """Test StockIpo initialization with minimal parameters"""
    ipo = StockIpo(session=mock_session)

    assert ipo.name == NAME
    assert ipo.key == KEY
    assert ipo.source == SOURCE
    assert ipo.target == TARGET
    assert isinstance(ipo.params, Params)
    assert isinstance(ipo.coolant, Coolant)
    assert isinstance(ipo.paginate, Paginate)
    assert isinstance(ipo.retry, Retry)


def test_ipo_init_with_params_dict(mock_session):
    """Test StockIpo initialization with params as dict"""
    params = {"start_date": "20230101", "end_date": "20231231"}
    ipo = StockIpo(session=mock_session, params=params)

    assert ipo.params.start_date == "20230101"
    assert ipo.params.end_date == "20231231"


def test_ipo_init_with_params_object(mock_session):
    """Test StockIpo initialization with Params object"""
    params = Params(year="2023")
    ipo = StockIpo(session=mock_session, params=params)

    assert ipo.params.year == "2023"


def test_ipo_init_with_year_param(mock_session):
    """Test StockIpo initialization with year parameter"""
    ipo = StockIpo(session=mock_session, params={"year": "2023"})

    assert ipo.params.year == "2023"


def test_ipo_init_with_trade_date_param(mock_session):
    """Test StockIpo initialization with trade_date parameter"""
    ipo = StockIpo(session=mock_session, params={"trade_date": "20230315"})

    assert ipo.params.trade_date == "20230315"


def test_ipo_init_with_cache_bool_true(mock_session):
    """Test StockIpo initialization with cache=True"""
    ipo = StockIpo(session=mock_session, cache=True)

    assert ipo.cache is not None
    assert isinstance(ipo.cache, Cache)


def test_ipo_init_with_cache_bool_false(mock_session):
    """Test StockIpo initialization with cache=False"""
    ipo = StockIpo(session=mock_session, cache=False)

    assert ipo.cache is None


def test_ipo_init_with_cache_dict(mock_session):
    """Test StockIpo initialization with cache as dict"""
    cache_config = {"directory": "/tmp/cache"}
    ipo = StockIpo(session=mock_session, cache=cache_config)

    assert ipo.cache is not None
    assert isinstance(ipo.cache, Cache)


def test_ipo_init_default_paginate_limit(mock_session):
    """Test StockIpo sets default paginate pagesize to 2000 and pagelimit to 5"""
    ipo = StockIpo(session=mock_session)

    assert ipo.paginate.pagesize == PAGINATE.pagesize
    assert ipo.paginate.pagelimit == PAGINATE.pagelimit


def test_ipo_init_with_all_params(mock_session):
    """Test StockIpo initialization with all parameters"""
    ipo = StockIpo(
        session=mock_session,
        params={"year": "2023"},
        coolant={"interval": 1.0},
        retry={"max_retries": 3},
        cache=True,
    )

    assert ipo.name == NAME
    assert ipo.params.year == "2023"
    assert ipo.cache is not None
    assert ipo.paginate.pagesize == PAGINATE.pagesize
    assert ipo.paginate.pagelimit == PAGINATE.pagelimit


def test_ipo_constants():
    """Test that constants are properly defined"""
    assert NAME == "stockipo"
    assert KEY == "/tushare/stockipo"
    assert SOURCE is not None
    assert TARGET is not None


# ============================================================================
# Transform Method Tests
# ============================================================================


def test_ipo_transform_basic(mock_session, sample_source_data):
    """Test basic data transformation"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    assert not result.empty
    assert len(result) == 3
    assert "code" in result.columns
    assert "name" in result.columns
    assert "ipo_date" in result.columns


def test_ipo_transform_code_mapping(mock_session, sample_source_data):
    """Test that ts_code is mapped to code"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    assert result["code"].tolist() == ["688001.SH", "688002.SH", "688003.SH"]


def test_ipo_transform_name_mapping(mock_session, sample_source_data):
    """Test that name is preserved"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    assert "华兴源创" in result["name"].values


def test_ipo_transform_ipo_date_format(mock_session, sample_source_data):
    """Test that ipo_date is converted from YYYYMMDD to YYYY-MM-DD"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    assert result["ipo_date"].tolist() == ["2019-06-27", "2019-06-28", "2019-07-10"]


def test_ipo_transform_ipo_datecode_preserved(mock_session, sample_source_data):
    """Test that ipo_datecode preserves original format"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    assert result["ipo_datecode"].tolist() == ["20190627", "20190628", "20190710"]


def test_ipo_transform_issue_date_format(mock_session, sample_source_data):
    """Test that issue_date is converted from YYYYMMDD to YYYY-MM-DD"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    assert result["issue_date"].tolist() == ["2019-07-10", "2019-07-12", "2019-07-22"]


def test_ipo_transform_issue_datecode_preserved(mock_session, sample_source_data):
    """Test that issue_datecode preserves original format"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    assert result["issue_datecode"].tolist() == ["20190710", "20190712", "20190722"]


def test_ipo_transform_numeric_conversions(mock_session, sample_source_data):
    """Test numeric field conversions"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    assert result["amount"].dtype in [float, "float64"]
    assert result["price"].dtype in [float, "float64"]
    assert result["pe"].dtype in [float, "float64"]


def test_ipo_transform_empty_dataframe(mock_session):
    """Test transform with empty DataFrame"""
    ipo = StockIpo(session=mock_session)
    empty_df = pd.DataFrame()
    result = ipo.transform(empty_df)

    assert result.empty
    assert list(result.columns) == TARGET.list_column_names()


def test_ipo_transform_none_input(mock_session):
    """Test transform with None input"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(None)

    assert result.empty
    assert list(result.columns) == TARGET.list_column_names()


def test_ipo_transform_handles_invalid_dates(mock_session):
    """Test transform handles invalid date formats"""
    ipo = StockIpo(session=mock_session)
    data = pd.DataFrame(
        {
            "ts_code": ["688001.SH"],
            "sub_code": ["787001"],
            "name": ["Test"],
            "ipo_date": ["invalid"],
            "issue_date": ["invalid"],
            "amount": [1000.0],
            "market_amount": [500.0],
            "price": [10.0],
            "pe": [20.0],
            "limit_amount": [5.0],
            "funds": [1.0],
            "ballot": [0.05],
        }
    )

    result = ipo.transform(data)
    assert pd.isna(result["ipo_date"].iloc[0]) or result["ipo_date"].iloc[0] == "NaT"


def test_ipo_transform_removes_duplicates(mock_session):
    """Test that transform removes duplicate rows"""
    ipo = StockIpo(session=mock_session)
    data = pd.DataFrame(
        {
            "ts_code": ["688001.SH", "688001.SH"],  # Duplicate
            "sub_code": ["787001", "787001"],
            "name": ["Test", "Test"],
            "ipo_date": ["20190627", "20190627"],
            "issue_date": ["20190710", "20190710"],
            "amount": [1000.0, 1000.0],
            "market_amount": [500.0, 500.0],
            "price": [10.0, 10.0],
            "pe": [20.0, 20.0],
            "limit_amount": [5.0, 5.0],
            "funds": [1.0, 1.0],
            "ballot": [0.05, 0.05],
        }
    )

    result = ipo.transform(data)
    assert len(result) == 1


def test_ipo_transform_sorts_by_code(mock_session, sample_source_data):
    """Test that result is sorted by code"""
    ipo = StockIpo(session=mock_session)
    shuffled = sample_source_data.sample(frac=1).reset_index(drop=True)
    result = ipo.transform(shuffled)

    assert result["code"].tolist() == sorted(result["code"].tolist())


def test_ipo_transform_resets_index(mock_session, sample_source_data):
    """Test that result has reset index"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    assert result.index.tolist() == list(range(len(result)))


def test_ipo_transform_only_target_columns(mock_session, sample_source_data):
    """Test that only target columns are in result"""
    ipo = StockIpo(session=mock_session)
    result = ipo.transform(sample_source_data)

    expected_cols = set(TARGET.list_column_names())
    actual_cols = set(result.columns)
    assert actual_cols == expected_cols


# ============================================================================
# _run Method Tests
# ============================================================================


def test_ipo_run_with_cache_hit(mock_session):
    """Test _run returns cached data when available"""
    ipo = StockIpo(session=mock_session, cache=True)

    cached_df = pd.DataFrame({"code": ["688001.SH"]})
    ipo.cache.set(ipo.params.identifier, cached_df)

    result = ipo._run()

    assert result.equals(cached_df)


def test_ipo_run_basic_date_range(mock_session, sample_source_data):
    """Test _run with start_date and end_date"""
    ipo = StockIpo(session=mock_session, params={"start_date": "20190101", "end_date": "20191231"})

    with patch.object(ipo, "_fetchall", return_value=sample_source_data):
        result = ipo._run()

        assert not result.empty
        ipo._fetchall.assert_called_once()
        call_kwargs = ipo._fetchall.call_args[1]
        assert call_kwargs["start_date"] == "20190101"
        assert call_kwargs["end_date"] == "20191231"


def test_ipo_run_with_year_param(mock_session, sample_source_data):
    """Test _run converts year to start/end date"""
    ipo = StockIpo(session=mock_session, params={"year": "2023"})

    with patch.object(ipo, "_fetchall", return_value=sample_source_data):
        ipo._run()

        call_kwargs = ipo._fetchall.call_args[1]
        assert call_kwargs["start_date"] == "20230101"
        assert call_kwargs["end_date"] == "20231231"
        assert "year" not in call_kwargs


def test_ipo_run_with_year_param_int(mock_session, sample_source_data):
    """Test _run handles year as integer"""
    ipo = StockIpo(session=mock_session, params={"year": 2023})

    with patch.object(ipo, "_fetchall", return_value=sample_source_data):
        ipo._run()

        call_kwargs = ipo._fetchall.call_args[1]
        assert call_kwargs["start_date"] == "20230101"
        assert call_kwargs["end_date"] == "20231231"


def test_ipo_run_adds_fields_param(mock_session, sample_source_data):
    """Test _run adds fields parameter if not provided"""
    ipo = StockIpo(session=mock_session)

    with patch.object(ipo, "_fetchall", return_value=sample_source_data):
        ipo._run()

        call_kwargs = ipo._fetchall.call_args[1]
        assert "fields" in call_kwargs


def test_ipo_run_preserves_fields_param(mock_session, sample_source_data):
    """Test _run preserves existing fields parameter"""
    custom_fields = "ts_code,name,ipo_date"
    ipo = StockIpo(session=mock_session, params={"fields": custom_fields})

    with patch.object(ipo, "_fetchall", return_value=sample_source_data):
        ipo._run()

        # Fields should be in the call
        call_kwargs = ipo._fetchall.call_args[1]
        assert "fields" in call_kwargs


def test_ipo_run_sets_cache(mock_session, sample_source_data):
    """Test _run saves result to cache"""
    ipo = StockIpo(session=mock_session, cache=True)

    with patch.object(ipo, "_fetchall", return_value=sample_source_data):
        ipo._run()

        cached = ipo.cache.get(ipo.params.identifier)
        assert cached is not None


def test_ipo_run_calls_transform(mock_session, sample_source_data):
    """Test _run calls transform method"""
    ipo = StockIpo(session=mock_session)

    with patch.object(ipo, "_fetchall", return_value=sample_source_data):
        with patch.object(ipo, "transform", return_value=sample_source_data) as mock_transform:
            ipo._run()

            mock_transform.assert_called_once()


# ============================================================================
# list_codes Method Tests
# ============================================================================


def test_ipo_list_codes_basic(mock_session, sample_source_data):
    """Test list_codes returns list of IPO codes"""
    ipo = StockIpo(session=mock_session, cache=True)

    transformed = ipo.transform(sample_source_data)
    ipo.cache.set(ipo.params.identifier, transformed)

    codes = ipo.list_codes()

    assert isinstance(codes, list)
    assert len(codes) == 3
    assert "688001.SH" in codes


def test_ipo_list_codes_unique(mock_session):
    """Test list_codes returns unique codes"""
    ipo = StockIpo(session=mock_session, cache=True)

    df = pd.DataFrame(
        {
            "code": ["688001.SH", "688001.SH", "688002.SH"],
        }
    )
    ipo.cache.set(ipo.params.identifier, df)

    codes = ipo.list_codes()

    assert len(codes) == 2


def test_ipo_list_codes_sorted(mock_session, sample_source_data):
    """Test list_codes returns sorted list"""
    ipo = StockIpo(session=mock_session, cache=True)

    transformed = ipo.transform(sample_source_data)
    ipo.cache.set(ipo.params.identifier, transformed)

    codes = ipo.list_codes()

    assert codes == sorted(codes)


def test_ipo_list_codes_calls_run_when_not_cached(mock_session, sample_source_data):
    """Test list_codes calls run() when data not in cache"""
    ipo = StockIpo(session=mock_session)

    with patch.object(ipo, "run", return_value=ipo.transform(sample_source_data)):
        codes = ipo.list_codes()

        ipo.run.assert_called_once()
        assert len(codes) == 3


# ============================================================================
# list_names Method Tests
# ============================================================================


def test_ipo_list_names_basic(mock_session, sample_source_data):
    """Test list_names returns list of IPO names"""
    ipo = StockIpo(session=mock_session, cache=True)

    transformed = ipo.transform(sample_source_data)
    ipo.cache.set(ipo.params.identifier, transformed)

    names = ipo.list_names()

    assert isinstance(names, list)
    assert len(names) == 3
    assert "华兴源创" in names


def test_ipo_list_names_sorted(mock_session, sample_source_data):
    """Test list_names returns sorted list"""
    ipo = StockIpo(session=mock_session, cache=True)

    transformed = ipo.transform(sample_source_data)
    ipo.cache.set(ipo.params.identifier, transformed)

    names = ipo.list_names()

    assert names == sorted(names)


def test_ipo_list_names_unique(mock_session):
    """Test list_names returns unique names"""
    ipo = StockIpo(session=mock_session, cache=True)

    df = pd.DataFrame(
        {
            "name": ["华兴源创", "华兴源创", "睿创微纳"],
        }
    )
    ipo.cache.set(ipo.params.identifier, df)

    names = ipo.list_names()

    assert len(names) == 2


def test_ipo_list_names_calls_run_when_not_cached(mock_session, sample_source_data):
    """Test list_names calls run() when data not in cache"""
    ipo = StockIpo(session=mock_session)

    with patch.object(ipo, "run", return_value=ipo.transform(sample_source_data)):
        names = ipo.list_names()

        ipo.run.assert_called_once()
        assert len(names) == 3


# ============================================================================
# Integration Tests
# ============================================================================


def test_ipo_full_workflow(mock_session, sample_source_data):
    """Test complete workflow from initialization to data retrieval"""
    ipo = StockIpo(
        session=mock_session,
        params={"year": "2019"},
        cache=True,
    )

    with patch.object(ipo, "_fetchall", return_value=sample_source_data):
        result = ipo.run()

        assert not result.empty
        assert "code" in result.columns

        codes = ipo.list_codes()
        names = ipo.list_names()

        assert len(codes) > 0
        assert len(names) > 0


def test_ipo_cache_persistence(mock_session, sample_source_data):
    """Test that cache persists across method calls"""
    ipo = StockIpo(session=mock_session, cache=True)

    with patch.object(ipo, "_load_cache", return_value=None) as mock_load:
        with patch.object(ipo, "_fetchall", return_value=sample_source_data) as mock_fetch:
            # First run - fetches data and caches it
            result1 = ipo.run()
            assert mock_fetch.call_count == 1
            assert mock_load.call_count == 1

            # Second run - _load_cache still returns None, so _fetchall called again
            result2 = ipo.run()
            assert mock_fetch.call_count == 2  # Called again
            assert mock_load.call_count == 2

            pd.testing.assert_frame_equal(result1, result2)


def test_ipo_params_identifier_uniqueness(mock_session):
    """Test that different params produce different cache keys"""
    ipo1 = StockIpo(session=mock_session, params={"year": "2022"}, cache=True)
    ipo2 = StockIpo(session=mock_session, params={"year": "2023"}, cache=True)

    assert ipo1.params.identifier != ipo2.params.identifier


# ============================================================================
# Edge Case Tests
# ============================================================================


def test_ipo_empty_result_handling(mock_session):
    """Test handling of empty API results"""
    ipo = StockIpo(session=mock_session)

    empty_df = pd.DataFrame()
    with patch.object(ipo, "_fetchall", return_value=empty_df):
        result = ipo._run()

        assert result.empty
        assert list(result.columns) == TARGET.list_column_names()


def test_ipo_large_dataset_handling(mock_session):
    """Test handling of large datasets"""
    ipo = StockIpo(session=mock_session)

    large_data = pd.DataFrame(
        {
            "ts_code": [f"{i:06d}.SH" for i in range(2000)],
            "sub_code": [f"78{i:04d}" for i in range(2000)],
            "name": [f"Stock {i}" for i in range(2000)],
            "ipo_date": ["20230101"] * 2000,
            "issue_date": ["20230115"] * 2000,
            "amount": [1000.0] * 2000,
            "market_amount": [500.0] * 2000,
            "price": [10.0] * 2000,
            "pe": [20.0] * 2000,
            "limit_amount": [5.0] * 2000,
            "funds": [1.0] * 2000,
            "ballot": [0.05] * 2000,
        }
    )

    result = ipo.transform(large_data)

    assert len(result) == 2000
    assert not result.empty


def test_ipo_special_characters_in_data(mock_session):
    """Test handling of special characters in IPO data"""
    ipo = StockIpo(session=mock_session)

    data = pd.DataFrame(
        {
            "ts_code": ["688001.SH"],
            "sub_code": ["787001"],
            "name": ["股票名称（中文）& Special <Chars>"],
            "ipo_date": ["20230101"],
            "issue_date": ["20230115"],
            "amount": [1000.0],
            "market_amount": [500.0],
            "price": [10.0],
            "pe": [20.0],
            "limit_amount": [5.0],
            "funds": [1.0],
            "ballot": [0.05],
        }
    )

    result = ipo.transform(data)

    assert len(result) == 1
    assert "股" in result["name"].values[0] or "Special" in result["name"].values[0]


def test_ipo_without_cache(mock_session, sample_source_data):
    """Test StockIpo works correctly without cache"""
    ipo = StockIpo(session=mock_session, cache=False)

    assert ipo.cache is None

    with patch.object(ipo, "_fetchall", return_value=sample_source_data):
        result = ipo.run()

        assert not result.empty

        codes = ipo.list_codes()
        names = ipo.list_names()

        assert len(codes) > 0
        assert len(names) > 0


def test_ipo_missing_numeric_values(mock_session):
    """Test handling of missing numeric values"""
    ipo = StockIpo(session=mock_session)

    data = pd.DataFrame(
        {
            "ts_code": ["688001.SH"],
            "sub_code": ["787001"],
            "name": ["Test Stock"],
            "ipo_date": ["20230101"],
            "issue_date": ["20230115"],
            "amount": [None],
            "market_amount": [None],
            "price": [None],
            "pe": [None],
            "limit_amount": [None],
            "funds": [None],
            "ballot": [None],
        }
    )

    result = ipo.transform(data)

    assert len(result) == 1
    assert pd.isna(result["amount"].iloc[0])
    assert pd.isna(result["price"].iloc[0])



from __future__ import annotations

from typing import Any, Dict, Optional

import pandas as pd

from xfintech.data.common.cache import Cache
from xfintech.data.common.coolant import Coolant
from xfintech.data.common.params import Params
from xfintech.data.common.retry import Retry
from xfintech.data.job import JobHouse
from xfintech.data.source.tushare.job import TushareJob
from xfintech.data.source.tushare.session.session import Session
from xfintech.data.source.tushare.stock.companybusiness.constant import (
    KEY,
    NAME,
    PAGINATE,
    SOURCE,
    TARGET,
)


@JobHouse.register(KEY, alias=KEY)
class CompanyBusiness(TushareJob):
    """
    描述:
    - 获取A股上市公司主营业务构成数据（个股级别数据，需要指定ts_code）
    - API文档: https://tushare.pro/document/2?doc_id=81
    - SCALE: CrossSection & Individual
    - TYPE: Partitioned
    - PAGINATE: 9000 rows / 1000 pages

    属性:
    - name: str, 作业名称 'companybusiness'。
    - key: str, 作业键 '/tushare/companybusiness'。
    - session: Session, Tushare会话对象。
    - source: TableInfo, 源表信息（Tushare原始格式）。
    - target: TableInfo, 目标表信息（转换后格式）。
    - params: Params, 查询参数。
        - ts_code: str, 必需, 股票代码（如'000001.SZ'）
        - year: str, 可选, 报告年度（YYYY）
        - start_date: str, 可选, 报告期开始日期（YYYYMMDD）
        - end_date: str, 可选, 报告期结束日期（YYYYMMDD）
        - period: str, 可选, 报告期（YYYYMMDD）
        - type: str, 可选, 类型：P按产品 D按地区 I按行业
    - coolant: Coolant, 请求冷却控制。
    - paginate: Paginate, 分页控制（pagesize=9000, pagelimit=1000）。
    - retry: Retry, 重试策略。
    - cache: Cache, 缓存管理。

    方法:
    - run(): 执行作业，返回主营业务构成DataFrame。
    - _run(): 内部执行逻辑，处理参数并调用API。
    - transform(data): 转换数据格式，将源格式转为目标格式。

    例子:
    ```python
    from xfintech.data.source.tushare.session import Session
    from xfintech.data.source.tushare.stock.companybusiness import CompanyBusiness

    session = Session(credential="your_token")
    business = CompanyBusiness(
        session=session,
        params={
            "ts_code": "000001.SZ",
            "period": "20201231",
            "type": "P",
        },
    )
    df = business.run()
    ```
    """

    def __init__(
        self,
        session: Session,
        params: Optional[Params | Dict[str, Any]] = None,
        coolant: Optional[Coolant | Dict[str, Any]] = None,
        retry: Optional[Retry | Dict[str, Any]] = None,
        cache: Optional[Cache | Dict[str, str] | bool] = None,
    ) -> None:
        super().__init__(
            name=NAME,
            key=KEY,
            session=session,
            source=SOURCE,
            target=TARGET,
            params=params,
            coolant=coolant,
            paginate=PAGINATE,
            retry=retry,
            cache=cache,
        )

    def _run(self) -> pd.DataFrame:
        cached = self._load_cache()
        if cached is not None:
            return cached

        # Prepare payload dict
        payload = self.params.to_dict()
        payload = self._parse_date_params(
            payload,
            keys=["start_date", "end_date"],
        )
        payload = self._parse_string_params(
            payload,
            keys=["ts_code", "type"],
        )
        payload = self._parse_year_params(
            payload,
            key="year",
        )
        payload = self._parse_period_params(
            payload,
            key="period",
        )

        # Add fields if not provided
        if "fields" not in payload:
            fields = self.source.list_column_names()
            payload["fields"] = ",".join(fields)

        # Fetch data from API
        data = self._fetchall(
            api=self.connection.fina_mainbz_vip,
            **payload,
        )
        result = self.transform(data)
        self._save_cache(result)
        return result

    # Transform logic
    def transform(
        self,
        data: pd.DataFrame,
    ) -> pd.DataFrame:
        cols = self.target.list_column_names()
        if data is None or data.empty:
            return pd.DataFrame(columns=cols)

        out = data.copy()
        out["code"] = out["ts_code"].astype(str)
        out["datecode"] = out["end_date"].astype(str)

        # Convert date fields
        out["date"] = pd.to_datetime(
            out["end_date"],
            format="%Y%m%d",
            errors="coerce",
        ).dt.strftime("%Y-%m-%d")
        out["end_date"] = pd.to_datetime(
            out["end_date"],
            format="%Y%m%d",
            errors="coerce",
        ).dt.strftime("%Y-%m-%d")

        # Convert string fields
        string_fields = ["bz_item", "curr_type", "update_flag"]
        for field in string_fields:
            if field in out.columns:
                out[field] = out[field].astype(str)

        # Convert numeric fields
        numeric_fields = ["bz_sales", "bz_profit", "bz_cost"]
        for field in numeric_fields:
            if field in out.columns:
                out[field] = pd.to_numeric(
                    out[field],
                    errors="coerce",
                )

        # Ensure all target columns are present
        for col in cols:
            if col not in out.columns:
                out[col] = pd.NA

        # Finalize output
        out = out[cols].drop_duplicates()
        out = out.sort_values(by=["code"])
        out = out.reset_index(drop=True)
        self.markpoint("transform[OK]")
        return out



from __future__ import annotations

from xfintech.data.common.paginate import Paginate
from xfintech.fabric.column.info import ColumnInfo
from xfintech.fabric.column.kind import ColumnKind
from xfintech.fabric.table.info import TableInfo

PROVIDER = "tushare"
SOURCE_NAME = "dividend"
URL = "https://tushare.pro/document/2?doc_id=103"
ARGS = {
    "ts_code": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "TS代码",
    },
    "ann_date": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "公告日",
    },
    "record_date": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "股权登记日期",
    },
    "ex_date": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "除权除息日",
    },
    "imp_ann_date": {
        "type": ColumnKind.STRING,
        "required": "N",
        "desc": "实施公告日",
    },
}

# Exported constants
NAME = "stockdividend"
KEY = "/tushare/stockdividend"
PAGINATE = Paginate(
    pagesize=1000,
    pagelimit=1000,
)
SOURCE = TableInfo(
    desc="分红送股数据（Tushare格式）",
    meta={
        "provider": PROVIDER,
        "source": SOURCE_NAME,
        "url": URL,
        "args": ARGS,
        "type": "partitioned",
        "scale": "crosssection",
    },
    name=SOURCE_NAME,
    columns=[
        ColumnInfo(name="ts_code", kind=ColumnKind.STRING, desc="TS代码"),
        ColumnInfo(name="end_date", kind=ColumnKind.STRING, desc="分红年度"),
        ColumnInfo(name="ann_date", kind=ColumnKind.STRING, desc="预案公告日"),
        ColumnInfo(name="div_proc", kind=ColumnKind.STRING, desc="实施进度"),
        ColumnInfo(name="stk_div", kind=ColumnKind.FLOAT, desc="每股送转"),
        ColumnInfo(name="stk_bo_rate", kind=ColumnKind.FLOAT, desc="每股送股比例"),
        ColumnInfo(name="stk_co_rate", kind=ColumnKind.FLOAT, desc="每股转增比例"),
        ColumnInfo(name="cash_div", kind=ColumnKind.FLOAT, desc="每股分红（税后）"),
        ColumnInfo(name="cash_div_tax", kind=ColumnKind.FLOAT, desc="每股分红（税前）"),
        ColumnInfo(name="record_date", kind=ColumnKind.STRING, desc="股权登记日"),
        ColumnInfo(name="ex_date", kind=ColumnKind.STRING, desc="除权除息日"),
        ColumnInfo(name="pay_date", kind=ColumnKind.STRING, desc="派息日"),
        ColumnInfo(name="div_listdate", kind=ColumnKind.STRING, desc="红股上市日"),
        ColumnInfo(name="imp_ann_date", kind=ColumnKind.STRING, desc="实施公告日"),
        ColumnInfo(name="base_date", kind=ColumnKind.STRING, desc="基准日"),
        ColumnInfo(name="base_share", kind=ColumnKind.FLOAT, desc="基准股本（万）"),
    ],
)

TARGET = TableInfo(
    desc="分红送股数据（标准格式）",
    meta={
        "key": KEY,
        "name": NAME,
        "type": "partitioned",
        "scale": "crosssection",
    },
    name=NAME,
    columns=[
        ColumnInfo(name="code", kind=ColumnKind.STRING, desc="股票代码"),
        ColumnInfo(name="end_date", kind=ColumnKind.STRING, desc="分红年度"),
        ColumnInfo(name="ann_date", kind=ColumnKind.STRING, desc="预案公告日"),
        ColumnInfo(name="ann_datecode", kind=ColumnKind.STRING, desc="预案公告日代码"),
        ColumnInfo(name="div_proc", kind=ColumnKind.STRING, desc="实施进度"),
        ColumnInfo(name="stk_div", kind=ColumnKind.FLOAT, desc="每股送转"),
        ColumnInfo(name="stk_bo_rate", kind=ColumnKind.FLOAT, desc="每股送股比例"),
        ColumnInfo(name="stk_co_rate", kind=ColumnKind.FLOAT, desc="每股转增比例"),
        ColumnInfo(name="cash_div", kind=ColumnKind.FLOAT, desc="每股分红（税后）"),
        ColumnInfo(name="cash_div_tax", kind=ColumnKind.FLOAT, desc="每股分红（税前）"),
        ColumnInfo(name="record_date", kind=ColumnKind.STRING, desc="股权登记日"),
        ColumnInfo(name="record_datecode", kind=ColumnKind.STRING, desc="股权登记日代码"),
        ColumnInfo(name="ex_date", kind=ColumnKind.STRING, desc="除权除息日"),
        ColumnInfo(name="ex_datecode", kind=ColumnKind.STRING, desc="除权除息日代码"),
        ColumnInfo(name="pay_date", kind=ColumnKind.STRING, desc="派息日"),
        ColumnInfo(name="pay_datecode", kind=ColumnKind.STRING, desc="派息日代码"),
        ColumnInfo(name="div_listdate", kind=ColumnKind.STRING, desc="红股上市日"),
        ColumnInfo(name="div_listdatecode", kind=ColumnKind.STRING, desc="红股上市日代码"),
        ColumnInfo(name="imp_ann_date", kind=ColumnKind.STRING, desc="实施公告日"),
        ColumnInfo(name="imp_ann_datecode", kind=ColumnKind.STRING, desc="实施公告日代码"),
        ColumnInfo(name="base_date", kind=ColumnKind.STRING, desc="基准日"),
        ColumnInfo(name="base_datecode", kind=ColumnKind.STRING, desc="基准日代码"),
        ColumnInfo(name="base_share", kind=ColumnKind.FLOAT, desc="基准股本（万）"),
    ],
)



from __future__ import annotations

from datetime import date, datetime
from unittest.mock import MagicMock, patch

import pandas as pd
import pytest

from xfintech.data.source.tushare.stock.techindex import (
    KEY,
    NAME,
    PAGINATE,
    SOURCE,
    TARGET,
    TechIndex,
)
from xfintech.data.source.tushare.stock.techindex.constant import (
    _IN_BFQ_COLS,
    _IN_HFQ_COLS,
    _IN_MAIN_COLS,
    _IN_QFQ_COLS,
    _OUT_BA_COLS,
    _OUT_FA_COLS,
    _OUT_MAIN_COLS,
    _OUT_NA_COLS,
)


# ================= Fixtures =================
@pytest.fixture
def mock_session():
    """创建模拟的Tushare会话对象"""
    session = MagicMock()

    # Mock the connection object
    mock_connection = MagicMock()
    mock_connection.stk_factor_pro = MagicMock()
    session.connection = mock_connection

    return session


@pytest.fixture
def sample_tushare_data():
    """创建样本Tushare格式数据"""
    return pd.DataFrame(
        {
            "ts_code": ["000001.SZ", "000002.SZ"],
            "trade_date": ["20240101", "20240101"],
            "open": [10.5, 20.3],
            "high": [11.0, 21.0],
            "low": [10.2, 20.0],
            "close": [10.8, 20.8],
            "change": [0.3, 0.5],
            "pct_chg": [2.86, 2.46],
            "vol": [100000.0, 200000.0],
            "amount": [108000.0, 416000.0],
            "turnover_rate": [1.5, 2.3],
            "turnover_rate_f": [1.2, 2.0],
            "volume_ratio": [1.1, 1.3],
            "pe": [15.5, 18.2],
            "pe_ttm": [16.0, 18.5],
            "pb": [2.5, 3.2],
            "ps": [3.5, 4.2],
            "ps_ttm": [3.8, 4.5],
            "dv_ratio": [2.5, 3.0],
            "dv_ttm": [2.8, 3.2],
            "total_share": [1000000.0, 2000000.0],
            "float_share": [800000.0, 1600000.0],
            "free_share": [700000.0, 1400000.0],
            "total_mv": [10800000.0, 41600000.0],
            "circ_mv": [8640000.0, 33280000.0],
            "adj_factor": [1.0, 1.0],
            "downdays": [0.0, 1.0],
            "updays": [3.0, 2.0],
            "lowdays": [5.0, 10.0],
            "topdays": [15.0, 20.0],
            # BFQ columns
            "macd_bfq": [0.05, 0.08],
            "macd_dif_bfq": [0.03, 0.05],
            "macd_dea_bfq": [0.02, 0.03],
            # HFQ columns
            "macd_hfq": [0.06, 0.09],
            "macd_dif_hfq": [0.04, 0.06],
            "macd_dea_hfq": [0.02, 0.03],
            # QFQ columns
            "macd_qfq": [0.055, 0.085],
            "macd_dif_qfq": [0.035, 0.055],
            "macd_dea_qfq": [0.02, 0.03],
        }
    )


@pytest.fixture
def sample_transformed_data():
    """创建样本转换后数据"""
    return pd.DataFrame(
        {
            "code": ["000001.SZ", "000002.SZ"],
            "date": ["2024-01-01", "2024-01-01"],
            "datecode": ["20240101", "20240101"],
            "open": [10.5, 20.3],
            "high": [11.0, 21.0],
            "low": [10.2, 20.0],
            "close": [10.8, 20.8],
            "change": [0.3, 0.5],
            "percent_change": [2.86, 2.46],
            "volume": [100000.0, 200000.0],
            "amount": [108000.0, 416000.0],
        }
    )


# ================= 初始化测试 =================
class TestTechIndexInitialization:
    """测试TechIndex类的初始化"""

    def test_init_with_minimal_params(self, mock_session):
        """测试使用最少参数初始化"""
        job = TechIndex(session=mock_session)
        assert job.name == NAME
        assert job.key == KEY
        assert job.source == SOURCE
        assert job.target == TARGET

    def test_init_with_params_dict(self, mock_session):
        """测试使用字典参数初始化"""
        params = {"ts_code": "000001.SZ"}
        job = TechIndex(session=mock_session, params=params)
        assert job.params.get("ts_code") == "000001.SZ"

    def test_init_with_date_params(self, mock_session):
        """测试使用日期参数初始化"""
        params = {
            "ts_code": "000001.SZ",
            "start_date": "20240101",
            "end_date": "20240131",
        }
        job = TechIndex(session=mock_session, params=params)
        assert job.params.get("start_date") == "20240101"
        assert job.params.get("end_date") == "20240131"

    def test_init_with_cache_enabled(self, mock_session):
        """测试启用缓存初始化"""
        job = TechIndex(session=mock_session, cache=True)
        assert job.cache is not None

    def test_init_with_cache_disabled(self, mock_session):
        """测试禁用缓存初始化"""
        job = TechIndex(session=mock_session, cache=False)
        assert job.cache is None

    def test_paginate_settings(self, mock_session):
        """测试分页设置"""
        job = TechIndex(session=mock_session)
        assert job.paginate.pagesize == PAGINATE.pagesize
        assert job.paginate.pagelimit == PAGINATE.pagelimit


# ================= 转换测试 =================
class TestTechIndexTransform:
    """测试TechIndex的数据转换功能"""

    def test_transform_main_with_valid_data(self, mock_session, sample_tushare_data):
        """测试主数据转换"""
        job = TechIndex(session=mock_session)
        result = job._transform_main(sample_tushare_data)

        assert not result.empty
        assert len(result) == 2
        assert "code" in result.columns
        assert "date" in result.columns
        assert "datecode" in result.columns
        assert result["code"].iloc[0] == "000001.SZ"
        assert result["date"].iloc[0] == "2024-01-01"
        assert result["datecode"].iloc[0] == "20240101"

    def test_transform_main_with_empty_data(self, mock_session):
        """测试空数据的主数据转换"""
        job = TechIndex(session=mock_session)
        result = job._transform_main(pd.DataFrame())

        assert result.empty
        assert len(result.columns) == len(_OUT_MAIN_COLS)

    def test_transform_na_with_valid_data(self, mock_session, sample_tushare_data):
        """测试不复权因子转换"""
        job = TechIndex(session=mock_session)
        result = job._transform_na(sample_tushare_data)

        assert not result.empty
        assert len(result) == 2
        assert "open" in result.columns
        assert "macd" in result.columns
        assert result["macd"].iloc[0] == 0.05

    def test_transform_na_with_empty_data(self, mock_session):
        """测试空数据的不复权因子转换"""
        job = TechIndex(session=mock_session)
        result = job._transform_na(pd.DataFrame())

        assert result.empty
        assert len(result.columns) == len(_OUT_NA_COLS)

    def test_transform_ba_with_valid_data(self, mock_session, sample_tushare_data):
        """测试后复权因子转换"""
        job = TechIndex(session=mock_session)
        result = job._transform_ba(sample_tushare_data)

        assert not result.empty
        assert len(result) == 2
        # BA transform adds ba_ prefix
        assert "ba_macd" in result.columns
        assert result["ba_macd"].iloc[0] == 0.06

    def test_transform_ba_with_empty_data(self, mock_session):
        """测试空数据的后复权因子转换"""
        job = TechIndex(session=mock_session)
        result = job._transform_ba(pd.DataFrame())

        assert result.empty
        assert len(result.columns) == len(_OUT_BA_COLS)

    def test_transform_fa_with_valid_data(self, mock_session, sample_tushare_data):
        """测试前复权因子转换"""
        job = TechIndex(session=mock_session)
        result = job._transform_fa(sample_tushare_data)

        assert not result.empty
        assert len(result) == 2
        # FA transform adds fa_ prefix
        assert "fa_macd" in result.columns
        assert result["fa_macd"].iloc[0] == 0.055

    def test_transform_fa_with_empty_data(self, mock_session):
        """测试空数据的前复权因子转换"""
        job = TechIndex(session=mock_session)
        result = job._transform_fa(pd.DataFrame())

        assert result.empty
        assert len(result.columns) == len(_OUT_FA_COLS)

    def test_transform_complete_with_valid_data(self, mock_session, sample_tushare_data):
        """测试完整数据转换"""
        job = TechIndex(session=mock_session)
        result = job.transform(sample_tushare_data)

        assert not result.empty
        assert len(result) == 2
        # Check all column groups are present
        assert "code" in result.columns  # main
        assert "open" in result.columns  # na
        assert "ba_macd" in result.columns  # ba (with prefix)
        assert "fa_macd" in result.columns  # fa (with prefix)
        # Total columns should match target
        assert len(result.columns) == len(TARGET.list_column_names())

    def test_transform_complete_with_empty_data(self, mock_session):
        """测试空数据的完整转换"""
        job = TechIndex(session=mock_session)
        result = job.transform(pd.DataFrame())

        assert result.empty
        assert len(result.columns) == len(TARGET.list_column_names())

    def test_transform_preserves_column_order(self, mock_session, sample_tushare_data):
        """测试转换保持列顺序"""
        job = TechIndex(session=mock_session)
        result = job.transform(sample_tushare_data)

        expected_columns = TARGET.list_column_names()
        assert list(result.columns) == expected_columns


# ================= Run方法测试 =================
class TestTechIndexRun:
    """测试TechIndex的run方法"""

    def test_run_with_ts_code(self, mock_session, sample_tushare_data):
        """测试使用股票代码运行"""
        job = TechIndex(session=mock_session, params={"ts_code": "000001.SZ"}, cache=False)

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.run()
            assert isinstance(result, pd.DataFrame)
            assert not result.empty

    def test_run_with_trade_date_string(self, mock_session, sample_tushare_data):
        """测试使用字符串交易日期运行"""
        job = TechIndex(session=mock_session, params={"trade_date": "20240101"}, cache=False)

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.run()
            assert isinstance(result, pd.DataFrame)

    def test_run_with_trade_date_datetime(self, mock_session, sample_tushare_data):
        """测试使用datetime交易日期运行"""
        job = TechIndex(
            session=mock_session,
            params={"trade_date": datetime(2024, 1, 1)},
            cache=False,
        )

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.run()
            assert isinstance(result, pd.DataFrame)

    def test_run_with_trade_date_date(self, mock_session, sample_tushare_data):
        """测试使用date交易日期运行"""
        job = TechIndex(session=mock_session, params={"trade_date": date(2024, 1, 1)}, cache=False)

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.run()
            assert isinstance(result, pd.DataFrame)

    def test_run_with_date_range_string(self, mock_session, sample_tushare_data):
        """测试使用字符串日期范围运行"""
        job = TechIndex(
            session=mock_session,
            params={"start_date": "20240101", "end_date": "20240131"},
            cache=False,
        )

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.run()
            assert isinstance(result, pd.DataFrame)

    def test_run_with_date_range_datetime(self, mock_session, sample_tushare_data):
        """测试使用datetime日期范围运行"""
        job = TechIndex(
            session=mock_session,
            params={
                "start_date": datetime(2024, 1, 1),
                "end_date": datetime(2024, 1, 31),
            },
            cache=False,
        )

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.run()
            assert isinstance(result, pd.DataFrame)

    def test_run_with_date_range_date(self, mock_session, sample_tushare_data):
        """测试使用date日期范围运行"""
        job = TechIndex(
            session=mock_session,
            params={"start_date": date(2024, 1, 1), "end_date": date(2024, 1, 31)},
            cache=False,
        )

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.run()
            assert isinstance(result, pd.DataFrame)

    def test_run_with_mixed_date_formats(self, mock_session, sample_tushare_data):
        """测试使用混合日期格式运行"""
        job = TechIndex(
            session=mock_session,
            params={"start_date": date(2024, 1, 1), "end_date": "20240131"},
            cache=False,
        )

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.run()
            assert isinstance(result, pd.DataFrame)

    def test_run_returns_dataframe(self, mock_session, sample_tushare_data):
        """测试run方法返回DataFrame类型"""
        job = TechIndex(session=mock_session, cache=False)

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.run()
            assert isinstance(result, pd.DataFrame)
            assert len(result) > 0


# ================= 缓存测试 =================
class TestTechIndexCache:
    """测试TechIndex的缓存功能"""

    def test_cache_disabled_by_default(self, mock_session):
        """测试默认不启用缓存"""
        job = TechIndex(session=mock_session, cache=False)
        assert job.cache is None

    def test_cache_load_returns_none_when_disabled(self, mock_session):
        """测试禁用缓存时_load_cache返回None"""
        job = TechIndex(session=mock_session, cache=False)
        result = job._load_cache()
        assert result is None

    def test_cache_save_does_nothing_when_disabled(self, mock_session, sample_transformed_data):
        """测试禁用缓存时_save_cache不执行操作"""
        job = TechIndex(session=mock_session, cache=False)
        # Should not raise any errors
        job._save_cache(sample_transformed_data)

    def test_run_with_cache_enabled(self, mock_session, sample_tushare_data):
        """测试启用缓存运行"""
        job = TechIndex(
            session=mock_session,
            params={"ts_code": "000001.SZ"},
            cache={"store": "memory"},
        )

        with (
            patch.object(job, "_fetchall", return_value=sample_tushare_data),
            patch.object(job, "_load_cache", return_value=None),
            patch.object(job, "_save_cache") as mock_save,
        ):
            result = job.run()
            assert isinstance(result, pd.DataFrame)
            mock_save.assert_called_once()


# ================= Slice方法测试 =================
class TestTechIndexSlice:
    """测试TechIndex的slice方法"""

    def test_slice_main_without_data(self, mock_session, sample_tushare_data):
        """测试slice_main方法不传入数据"""
        job = TechIndex(session=mock_session, cache=False)

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.slice_main()
            assert isinstance(result, pd.DataFrame)
            assert not result.empty
            assert len(result.columns) == len(_OUT_MAIN_COLS)
            assert "code" in result.columns
            assert "date" in result.columns

    def test_slice_main_with_data(self, mock_session, sample_tushare_data):
        """测试slice_main方法传入数据"""
        job = TechIndex(session=mock_session, cache=False)
        full_data = job.transform(sample_tushare_data)
        result = job.slice_main(data=full_data)

        assert isinstance(result, pd.DataFrame)
        assert not result.empty
        assert len(result.columns) == len(_OUT_MAIN_COLS)

    def test_slice_na_without_data(self, mock_session, sample_tushare_data):
        """测试slice_na方法不传入数据"""
        job = TechIndex(session=mock_session, cache=False)

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.slice_na()
            assert isinstance(result, pd.DataFrame)
            assert not result.empty
            assert len(result.columns) == len(_OUT_MAIN_COLS + _OUT_NA_COLS)
            assert "open" in result.columns
            assert "macd" in result.columns

    def test_slice_na_with_data(self, mock_session, sample_tushare_data):
        """测试slice_na方法传入数据"""
        job = TechIndex(session=mock_session, cache=False)
        full_data = job.transform(sample_tushare_data)
        result = job.slice_na(data=full_data)

        assert isinstance(result, pd.DataFrame)
        assert not result.empty
        assert len(result.columns) == len(_OUT_MAIN_COLS + _OUT_NA_COLS)

    def test_slice_ba_without_data(self, mock_session, sample_tushare_data):
        """测试slice_ba方法不传入数据"""
        job = TechIndex(session=mock_session, cache=False)

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.slice_ba()
            assert isinstance(result, pd.DataFrame)
            assert not result.empty
            assert len(result.columns) == len(_OUT_MAIN_COLS + _OUT_BA_COLS)
            assert "ba_macd" in result.columns

    def test_slice_ba_with_data(self, mock_session, sample_tushare_data):
        """测试slice_ba方法传入数据"""
        job = TechIndex(session=mock_session, cache=False)
        full_data = job.transform(sample_tushare_data)
        result = job.slice_ba(data=full_data)

        assert isinstance(result, pd.DataFrame)
        assert not result.empty
        assert len(result.columns) == len(_OUT_MAIN_COLS + _OUT_BA_COLS)

    def test_slice_fa_without_data(self, mock_session, sample_tushare_data):
        """测试slice_fa方法不传入数据"""
        job = TechIndex(session=mock_session, cache=False)

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            result = job.slice_fa()
            assert isinstance(result, pd.DataFrame)
            assert not result.empty
            assert len(result.columns) == len(_OUT_MAIN_COLS + _OUT_FA_COLS)
            assert "fa_macd" in result.columns

    def test_slice_fa_with_data(self, mock_session, sample_tushare_data):
        """测试slice_fa方法传入数据"""
        job = TechIndex(session=mock_session, cache=False)
        full_data = job.transform(sample_tushare_data)
        result = job.slice_fa(data=full_data)

        assert isinstance(result, pd.DataFrame)
        assert not result.empty
        assert len(result.columns) == len(_OUT_MAIN_COLS + _OUT_FA_COLS)

    def test_slice_with_empty_data(self, mock_session):
        """测试slice方法处理空数据"""
        job = TechIndex(session=mock_session, cache=False)
        empty_data = job.transform(pd.DataFrame())

        result_main = job.slice_main(data=empty_data)
        result_na = job.slice_na(data=empty_data)
        result_ba = job.slice_ba(data=empty_data)
        result_fa = job.slice_fa(data=empty_data)

        assert result_main.empty
        assert result_na.empty
        assert result_ba.empty
        assert result_fa.empty


# ================= 集成测试 =================
class TestTechIndexIntegration:
    """TechIndex的集成测试"""

    def test_complete_workflow(self, mock_session, sample_tushare_data):
        """测试完整工作流程"""
        job = TechIndex(
            session=mock_session,
            params={"ts_code": "000001.SZ", "start_date": "20240101", "end_date": "20240131"},
            cache=False,
        )

        with patch.object(job, "_fetchall", return_value=sample_tushare_data):
            # 运行并获取完整数据
            full_result = job.run()
            assert isinstance(full_result, pd.DataFrame)
            assert not full_result.empty

            # 获取各部分数据
            main_result = job.slice_main(data=full_result)
            na_result = job.slice_na(data=full_result)
            ba_result = job.slice_ba(data=full_result)
            fa_result = job.slice_fa(data=full_result)

            # 验证各部分
            assert not main_result.empty
            assert not na_result.empty
            assert not ba_result.empty
            assert not fa_result.empty

            # 验证行数一致
            assert len(main_result) == len(na_result) == len(ba_result) == len(fa_result)

    def test_module_constants(self):
        """测试模块常量"""
        assert NAME == "techindex"
        assert KEY == "/tushare/techindex"
        assert PAGINATE.pagesize == 10000
        assert PAGINATE.pagelimit == 1000

    def test_source_table_structure(self):
        """测试源表结构"""
        assert SOURCE.desc == "A股股票技术面因子数据（Tushare格式）"
        assert SOURCE.meta["provider"] == "tushare"
        assert SOURCE.meta["source"] == "stk_factor_pro"
        total_cols = len(_IN_MAIN_COLS) + len(_IN_BFQ_COLS) + len(_IN_HFQ_COLS) + len(_IN_QFQ_COLS)
        assert len(SOURCE.columns) == total_cols

    def test_target_table_structure(self):
        """测试目标表结构"""
        assert TARGET.desc == "A股股票技术面因子数据（xfintech格式）"
        assert TARGET.meta["key"] == KEY
        assert TARGET.meta["name"] == NAME
        total_cols = len(_OUT_MAIN_COLS) + len(_OUT_NA_COLS) + len(_OUT_BA_COLS) + len(_OUT_FA_COLS)
        assert len(TARGET.columns) == total_cols



from __future__ import annotations

from datetime import date, datetime
from typing import Any, Callable, Dict, List, Optional

import pandas as pd

from xfintech.data.common.cache import Cache
from xfintech.data.common.coolant import Coolant
from xfintech.data.common.paginate import Paginate
from xfintech.data.common.params import Params
from xfintech.data.common.retry import Retry
from xfintech.data.job.job import Job
from xfintech.data.source.baostock.session.session import Session
from xfintech.fabric.table.info import TableInfo


class BaostockJob(Job):
    """
    描述:
    - Baostock 数据源作业类，继承自 Job 基类。
    - 专门用于从 Baostock API 获取金融数据。
    - 自动处理连接管理、数据获取、重试和缓存等功能。
    - 支持 Direct 模式（直接连接）和 Relay 模式（中继服务器）。
    - 自动检测 Direct/Relay 模式并处理 ResultSet 或 DataFrame。
    - 支持日期格式自动转换（YYYYMMDD → YYYY-MM-DD）。

    属性:
    - connection: Any, Baostock API 连接对象，从 session 中获取。
    - 继承 Job 类的所有属性（name, key, params, coolant, paginate, retry, cache, metric）。

    方法:
    - _resolve_connection(session): 从 session 解析并验证连接对象。
    - _fetchall(api, **params): 获取所有数据并自动处理 Direct/Relay 模式。
    - _parse_date_params(payload, keys): 解析日期参数，支持多种格式。
    - _parse_string_params(payload, keys): 解析字符串参数。

    例子:
    ```python
        from xfintech.data.source.baostock.job import BaostockJob
        from xfintech.data.source.baostock.session import Session

        # 创建 session
        session = Session()

        # 创建作业
        job = BaostockJob(
            name="minuteline",
            key="/baostock/minuteline",
            session=session,
            params={"code": "sh.600000", "start_date": "20240101"},
            paginate={"pagesize": 100, "pagelimit": 1000},
            retry={"retry": 3, "wait": 1},
            cache=True
        )

        # 在子类中实现 _run 方法来调用 _fetchall
        class MinutelineJob(BaostockJob):
            def _run(self):
                params = self.get_params()
                params = self._parse_date_params(params, ["start_date", "end_date"])
                return self._fetchall(
                    api=self.connection.query_history_k_data_plus,
                    **params
                )
    ```
    """

    def __init__(
        self,
        name: str,
        key: str,
        session: Session,
        source: Optional[TableInfo] = None,
        target: Optional[TableInfo] = None,
        params: Optional[Params | Dict[str, Any]] = None,
        coolant: Optional[Coolant | Dict[str, Any]] = None,
        paginate: Optional[Paginate | Dict[str, Any]] = None,
        retry: Optional[Retry | Dict[str, Any]] = None,
        cache: Optional[Cache | Dict[str, str] | bool] = None,
    ) -> None:
        super().__init__(
            name=name,
            key=key,
            source=source,
            target=target,
            params=params,
            coolant=coolant,
            paginate=paginate,
            retry=retry,
            cache=cache,
        )
        self.connection = self._resolve_connection(
            session,
        )

    def _resolve_connection(
        self,
        session: Session,
    ) -> Any:
        connection = getattr(session, "connection", None)
        if connection is None:
            msg = "No active connection found in session."
            raise ConnectionError(msg)
        self.markpoint("_resolve_connection[OK]")
        return connection

    def _parse_string_params(
        self,
        payload: Dict[str, Any],
        keys: List[str],
    ) -> Dict[str, Any]:
        payload = payload.copy()
        for key in keys:
            if key in payload:
                value = payload.get(key)
                if isinstance(value, str):
                    payload[key] = value
        return payload

    def _parse_date_params(
        self,
        payload: Dict[str, Any],
        keys: List[str],
    ) -> Dict[str, Any]:
        payload = payload.copy()
        for key in keys:
            if key in payload:
                value = payload.get(key)
                if isinstance(value, (date, datetime)):
                    payload[key] = value.strftime("%Y-%m-%d")
                if isinstance(value, date):
                    payload[key] = value.strftime("%Y-%m-%d")
                if isinstance(value, str):
                    if "-" in value:  # YYYY-MM-DD
                        payload[key] = value
                    else:  # YYYYMMDD
                        dt = datetime.strptime(value, "%Y%m%d")
                        payload[key] = dt.strftime("%Y-%m-%d")
        return payload

    def _parse_year_params(
        self,
        payload: Dict[str, Any],
        key: str,
    ) -> Dict[str, Any]:
        payload = payload.copy()
        if key not in payload:
            return payload
        else:
            year = str(payload.pop(key))
            if year.isdigit() and len(year) == 4:
                payload["start_date"] = f"{year}-01-01"
                payload["end_date"] = f"{year}-12-31"
            return payload

    def _fetchall(
        self,
        api: Callable,
        **params: Any,
    ) -> pd.DataFrame:
        """
        描述:
        - 使用迭代器获取所有数据并合并为单个 DataFrame。

        参数:
        - api: Callable, Baostock API 方法（如 connection.query_history_k_data_plus）
        - **params: Any, 传递给 API 方法的参数

        返回:
        - pd.DataFrame: 合并后的完整数据，如果没有数据或出错则返回空 DataFrame。

        说明:
        - Direct 模式：调用 api(**params) 返回 ResultSet，使用 get_data() 获取所有数据
        - Relay 模式：relay 在服务端执行完整的 get_data()，直接返回 DataFrame
        - 自动检测返回类型并处理
        - Baostock 默认每页 10,000 行 (BAOSTOCK_PER_PAGE_COUNT)
        - API 调用数 = ceil(总行数 / 10,000)

        """
        self.paginate.reset()
        result = api(**params)
        self.markpoint("_fetchall[api_called]")

        # Relay mode will return DataFrame directly
        if isinstance(result, pd.DataFrame):
            self.markpoint(f"_fetchall[relay_mode, rows={len(result)}]")
            return result

        # Direct mode will return ResultSet
        else:
            if result.error_code != "0":
                errcode = result.error_code
                errmsg = result.error_msg
                self.markpoint(f"_fetchall[ERROR: code={errcode}, msg={errmsg}]")
                return pd.DataFrame()
            data = result.get_data()
            self.markpoint(f"_fetchall[direct_mode, rows={len(data)}]")
            return data

    def _load_cache(
        self,
    ) -> Optional[pd.DataFrame]:
        if self.cache:
            cached = self.cache.get(self.params.identifier)
            if cached is not None:
                self.markpoint("load_cache[OK]")
                return cached
        return None

    def _save_cache(
        self,
        data: pd.DataFrame,
    ) -> None:
        if self.cache:
            self.markpoint("_save_cache[OK]")
            self.cache.set(self.params.identifier, data)
